@@ -15,39 +15,45 @@
 #include "copydb.h"
 #include "commandline.h"
 #include "env_utils.h"
+#include "ld_stream.h"
 #include "log.h"
-#include "parsing.h"
+#include "parsing_utils.h"
 #include "pgsql.h"
-#include "stream.h"
+#include "progress.h"
 #include "string_utils.h"
 #include "summary.h"
 
+#define PGCOPYDB_CLONE_GETOPTS_HELP \
+	"  --source                   Postgres URI to the source database\n" \
+	"  --target                   Postgres URI to the target database\n" \
+	"  --dir                      Work directory to use\n" \
+	"  --table-jobs               Number of concurrent COPY jobs to run\n" \
+	"  --index-jobs               Number of concurrent CREATE INDEX jobs to run\n" \
+	"  --split-tables-larger-than Same-table concurrency size threshold\n" \
+	"  --drop-if-exists           On the target database, clean-up from a previous run first\n" \
+	"  --roles                    Also copy roles found on source to target\n" \
+	"  --no-owner                 Do not set ownership of objects to match the original database\n" \
+	"  --no-acl                   Prevent restoration of access privileges (grant/revoke commands).\n" \
+	"  --no-comments              Do not output commands to restore comments\n" \
+	"  --skip-large-objects       Skip copying large objects (blobs)\n" \
+	"  --skip-extensions          Skip restoring extensions\n" \
+	"  --filters <filename>       Use the filters defined in <filename>\n" \
+	"  --restart                  Allow restarting when temp files exist already\n" \
+	"  --resume                   Allow resuming operations after a failure\n" \
+	"  --not-consistent           Allow taking a new snapshot on the source database\n" \
+	"  --snapshot                 Use snapshot obtained with pg_export_snapshot\n" \
+	"  --follow                   Implement logical decoding to replay changes\n" \
+	"  --slot-name                Use this Postgres replication slot name\n" \
+	"  --create-slot              Create the replication slot\n" \
+	"  --origin                   Use this Postgres replication origin node name\n" \
+	"  --endpos                   Stop replaying changes when reaching this LSN\n" \
+
 CommandLine clone_command =
 	make_command(
 		"clone",
 		"Clone an entire database from source to target",
 		" --source ... --target ... [ --table-jobs ... --index-jobs ... ] ",
-		"  --source              Postgres URI to the source database\n"
-		"  --target              Postgres URI to the target database\n"
-		"  --dir                 Work directory to use\n"
-		"  --table-jobs          Number of concurrent COPY jobs to run\n"
-		"  --index-jobs          Number of concurrent CREATE INDEX jobs to run\n"
-		"  --drop-if-exists      On the target database, clean-up from a previous run first\n"
-		"  --roles               Also copy roles found on source to target\n"
-		"  --no-owner            Do not set ownership of objects to match the original database\n"
-		"  --no-acl              Prevent restoration of access privileges (grant/revoke commands).\n"
-		"  --no-comments         Do not output commands to restore comments\n"
-		"  --skip-large-objects  Skip copying large objects (blobs)\n"
-		"  --filters <filename>  Use the filters defined in <filename>\n"
-		"  --restart             Allow restarting when temp files exist already\n"
-		"  --resume              Allow resuming operations after a failure\n"
-		"  --not-consistent      Allow taking a new snapshot on the source database\n"
-		"  --snapshot            Use snapshot obtained with pg_export_snapshot\n"
-		"  --follow              Implement logical decoding to replay changes\n"
-		"  --slot-name           Use this Postgres replication slot name\n"
-		"  --create-slot         Create the replication slot\n"
-		"  --origin              Use this Postgres replication origin node name\n"
-		"  --endpos              Stop replaying changes when reaching this LSN\n",
+		PGCOPYDB_CLONE_GETOPTS_HELP,
 		cli_copy_db_getopts,
 		cli_clone);
 
@@ -56,27 +62,7 @@ CommandLine fork_command =
 		"fork",
 		"Clone an entire database from source to target",
 		" --source ... --target ... [ --table-jobs ... --index-jobs ... ] ",
-		"  --source              Postgres URI to the source database\n"
-		"  --target              Postgres URI to the target database\n"
-		"  --dir                 Work directory to use\n"
-		"  --table-jobs          Number of concurrent COPY jobs to run\n"
-		"  --index-jobs          Number of concurrent CREATE INDEX jobs to run\n"
-		"  --drop-if-exists      On the target database, clean-up from a previous run first\n"
-		"  --roles               Also copy roles found on source to target\n"
-		"  --no-owner            Do not set ownership of objects to match the original database\n"
-		"  --no-acl              Prevent restoration of access privileges (grant/revoke commands).\n"
-		"  --no-comments         Do not output commands to restore comments\n"
-		"  --skip-large-objects  Skip copying large objects (blobs)\n"
-		"  --filters <filename>  Use the filters defined in <filename>\n"
-		"  --restart             Allow restarting when temp files exist already\n"
-		"  --resume              Allow resuming operations after a failure\n"
-		"  --not-consistent      Allow taking a new snapshot on the source database\n"
-		"  --snapshot            Use snapshot obtained with pg_export_snapshot\n"
-		"  --follow              Implement logical decoding to replay changes\n"
-		"  --slot-name           Use this Postgres replication slot name\n"
-		"  --create-slot         Create the replication slot\n"
-		"  --origin              Use this Postgres replication origin node name\n"
-		"  --endpos              Stop replaying changes when reaching this LSN\n",
+		PGCOPYDB_CLONE_GETOPTS_HELP,
 		cli_copy_db_getopts,
 		cli_clone);
 
@@ -148,13 +134,100 @@ cli_clone(int argc, char **argv)
 	 * First, we need to open a snapshot that we're going to re-use in all our
 	 * connections to the source database. When the --snapshot option has been
 	 * used, instead of exporting a new snapshot, we can just re-use it.
+	 *
+	 * When using PostgreSQL 9.6 logical decoding, we need to create our
+	 * replication slot and fetch the snapshot from that logical replication
+	 * command, it's the only way.
 	 */
-	if (!copydb_prepare_snapshot(&copySpecs))
+	bool snapshotExported = false;
+
+	if (copySpecs.follow)
+	{
+		TransactionSnapshot *snapshot = &(copySpecs.sourceSnapshot);
+		PGSQL *pgsql = &(snapshot->pgsql);
+
+		if (!pgsql_init(pgsql, copySpecs.source_pguri, PGSQL_CONN_SOURCE))
+		{
+			/* errors have already been logged */
+			exit(EXIT_CODE_SOURCE);
+		}
+
+		if (!pgsql_server_version(pgsql))
+		{
+			/* errors have already been logged */
+			exit(EXIT_CODE_SOURCE);
+		}
+
+		if (pgsql->pgversion_num < 100000)
+		{
+			if (!copydb_create_logical_replication_slot(&copySpecs,
+														streamSpecs.logrep_pguri,
+														streamSpecs.slotName))
+			{
+				/* errors have already been logged */
+				exit(EXIT_CODE_SOURCE);
+			}
+
+			snapshotExported = true;
+		}
+	}
+
+	/*
+	 * Make sure to export a snapshot if that's not been done just above.
+	 */
+	if (!snapshotExported && !copydb_prepare_snapshot(&copySpecs))
 	{
 		/* errors have already been logged */
 		exit(EXIT_CODE_INTERNAL_ERROR);
 	}
 
+	/* just keep track, even though we're not going to use this anymore */
+	snapshotExported = true;
+
+	/*
+	 * When --follow has been used, we start two subprocess (clone, follow).
+	 * Before doing that though, we want to make sure it was possible to setup
+	 * the source and target database for Change Data Capture: the wal2json
+	 * logical decoding plugin must be available on the source Postgres
+	 * instance, etc.
+	 */
+	if (copySpecs.follow)
+	{
+		/*
+		 * We want to make sure to use a private PGSQL client connection
+		 * instance when connecting to the source database now, as the main
+		 * connection is currently active holding a snapshot for the whole
+		 * process.
+		 */
+		CopyDataSpec setupSpecs = { 0 };
+		TransactionSnapshot snapshot = { 0 };
+
+		/* copy our structure wholesale */
+		setupSpecs = copySpecs;
+
+		/* ensure we use a new snapshot and connection in setupSpecs */
+		if (!copydb_copy_snapshot(&copySpecs, &snapshot))
+		{
+			/* errors have already been logged */
+			exit(EXIT_CODE_INTERNAL_ERROR);
+		}
+
+		setupSpecs.sourceSnapshot = snapshot;
+
+		/*
+		 * Now create the replication slot and the pgcopydb sentinel table on
+		 * the source database, and the origin (replication progress tracking)
+		 * on the target database.
+		 */
+		if (!stream_setup_databases(&setupSpecs,
+									streamSpecs.slotName,
+									streamSpecs.origin))
+		{
+			/* errors have already been logged */
+			exit(EXIT_CODE_INTERNAL_ERROR);
+		}
+	}
+
 	/*
 	 * Preparation and snapshot are now done, time to fork our two main worker
 	 * processes.
@@ -188,11 +261,72 @@ cli_clone(int argc, char **argv)
 		exit(EXIT_CODE_SOURCE);
 	}
 
+	/*
+	 * If we failed to do the clone parts (midway through, or entirely maybe),
+	 * we need to make it so that the follow sub-process isn't going to wait
+	 * forever to reach the apply mode and then the endpos. That will never
+	 * happen.
+	 */
+	if (!success)
+	{
+		log_warn("Failed to clone the source database, see above for details");
+
+		if (!copydb_fatal_exit())
+		{
+			/* errors have already been logged */
+			exit(EXIT_CODE_INTERNAL_ERROR);
+		}
+
+		exit(EXIT_CODE_INTERNAL_ERROR);
+	}
+
 	/* now wait until the follow process is finished */
 	if (copySpecs.follow)
 	{
 		success = success &&
 				  cli_clone_follow_wait_subprocess("follow", followPID);
+
+		/*
+		 * Now is a good time to reset the sequences on the target database to
+		 * match the state they are in at the moment on the source database.
+		 * Postgres logical decoding lacks support for syncing sequences.
+		 *
+		 * This step is implement as if running the following command:
+		 *
+		 *   $ pgcopydb copy sequences --resume --not-consistent
+		 *
+		 * The whole idea is to fetch the "new" current values of the
+		 * sequences, not the ones that were current when the main snapshot was
+		 * exported.
+		 */
+		CopyDataSpec seqSpecs = { 0 };
+
+		/* copy our structure wholesale */
+		seqSpecs = copySpecs;
+
+		/* then force some options such as --resume --not-consistent */
+		seqSpecs.restart = false;
+		seqSpecs.resume = true;
+		seqSpecs.consistent = false;
+		seqSpecs.section = DATA_SECTION_SET_SEQUENCES;
+
+		/* we don't want to re-use any snapshot */
+		TransactionSnapshot snapshot = { 0 };
+
+		seqSpecs.sourceSnapshot = snapshot;
+
+		/* fetch schema information from source catalogs, including filtering */
+		if (!copydb_fetch_schema_and_prepare_specs(&seqSpecs))
+		{
+			/* errors have already been logged */
+			exit(EXIT_CODE_TARGET);
+		}
+
+		if (!copydb_copy_all_sequences(&seqSpecs))
+		{
+			/* errors have already been logged */
+			exit(EXIT_CODE_INTERNAL_ERROR);
+		}
 	}
 
 	if (!success)
@@ -285,7 +419,7 @@ start_clone_process(CopyDataSpec *copySpecs, pid_t *pid)
 	{
 		case -1:
 		{
-			log_error("Failed to fork a subprocess to prefetch changes");
+			log_error("Failed to fork a subprocess to prefetch changes: %m");
 			return -1;
 		}
 
@@ -296,7 +430,8 @@ start_clone_process(CopyDataSpec *copySpecs, pid_t *pid)
 
 			if (!cloneDB(copySpecs))
 			{
-				/* errors have already been logged */
+				log_error("Failed to clone source database, "
+						  "see above for details");
 				exit(EXIT_CODE_SOURCE);
 			}
 
@@ -336,7 +471,10 @@ cloneDB(CopyDataSpec *copySpecs)
 	{
 		log_info("STEP 0: copy the source database roles");
 
-		if (!copydb_copy_roles(copySpecs))
+		if (!pg_copy_roles(&(copySpecs->pgPaths),
+						   copySpecs->source_pguri,
+						   copySpecs->target_pguri,
+						   copySpecs->dumpPaths.rolesFilename))
 		{
 			/* errors have already been logged */
 			return false;
@@ -355,8 +493,6 @@ cloneDB(CopyDataSpec *copySpecs)
 		return false;
 	}
 
-	log_info("STEP 2: restore the pre-data section to the target database");
-
 	/* make sure that we have our own process local connection */
 	TransactionSnapshot snapshot = { 0 };
 
@@ -376,14 +512,28 @@ cloneDB(CopyDataSpec *copySpecs)
 	}
 
 	/* fetch schema information from source catalogs, including filtering */
+
+	log_info("STEP 2: fetch source database tables, indexes, and sequences");
+
+	(void) summary_set_current_time(timings, TIMING_STEP_BEFORE_SCHEMA_FETCH);
+
 	if (!copydb_fetch_schema_and_prepare_specs(copySpecs))
 	{
 		/* errors have already been logged */
 		return false;
 	}
 
+	/* prepare the schema JSON file with all the migration details */
+	if (!copydb_prepare_schema_json_file(copySpecs))
+	{
+		/* errors have already been logged */
+		return false;
+	}
+
 	(void) summary_set_current_time(timings, TIMING_STEP_BEFORE_PREPARE_SCHEMA);
 
+	log_info("STEP 3: restore the pre-data section to the target database");
+
 	if (!copydb_target_prepare_schema(copySpecs))
 	{
 		/* errors have already been logged */
@@ -392,9 +542,10 @@ cloneDB(CopyDataSpec *copySpecs)
 
 	(void) summary_set_current_time(timings, TIMING_STEP_AFTER_PREPARE_SCHEMA);
 
-	log_info("STEP 3: copy data from source to target in sub-processes");
-	log_info("STEP 4: create indexes and constraints in parallel");
-	log_info("STEP 5: vacuum analyze each table");
+	log_info("STEP 4: copy data from source to target in %d sub-processes",
+			 copySpecs->tableJobs);
+
+	/* STEPs 5, 6, 7, 8, and 9 are printed when starting the sub-processes */
 
 	if (!copydb_copy_all_table_data(copySpecs))
 	{
@@ -405,7 +556,7 @@ cloneDB(CopyDataSpec *copySpecs)
 	/* close our snapshot: commit transaction and finish connection */
 	(void) copydb_close_snapshot(copySpecs);
 
-	log_info("STEP 7: restore the post-data section to the target database");
+	log_info("STEP 10: restore the post-data section to the target database");
 
 	(void) summary_set_current_time(timings, TIMING_STEP_BEFORE_FINALIZE_SCHEMA);
 
@@ -462,7 +613,7 @@ start_follow_process(CopyDataSpec *copySpecs, StreamSpecs *streamSpecs,
 	{
 		case -1:
 		{
-			log_error("Failed to fork a subprocess to prefetch changes");
+			log_error("Failed to fork a subprocess to prefetch changes: %m");
 			return -1;
 		}
 
@@ -494,23 +645,14 @@ start_follow_process(CopyDataSpec *copySpecs, StreamSpecs *streamSpecs,
 
 /*
  * followDB implements a logical decoding client for streaming changes from the
- * source database into the target database.
+ * source database into the target database. The stream_setup_databases() must
+ * have been called already so that the replication slot using wal2json is
+ * ready, the pgcopydb.sentinel table exists, and the target database
+ * replication origin has been created too.
  */
 static bool
 followDB(CopyDataSpec *copySpecs, StreamSpecs *streamSpecs)
 {
-	/*
-	 * Create the replication slot on the source database, and the origin
-	 * (replication progress tracking) on the target database.
-	 */
-	if (!stream_setup_databases(copySpecs,
-								streamSpecs->slotName,
-								streamSpecs->origin))
-	{
-		/* errors have already been logged */
-		return false;
-	}
-
 	/*
 	 * Remove the possibly still existing stream context files from
 	 * previous round of operations (--resume, etc). We want to make sure
@@ -564,12 +706,10 @@ static bool
 cli_clone_follow_wait_subprocess(const char *name, pid_t pid)
 {
 	bool exited = false;
-	bool success = true;
+	int returnCode = -1;
 
 	while (!exited)
 	{
-		int returnCode = -1;
-
 		if (!follow_wait_pid(pid, &exited, &returnCode))
 		{
 			/* errors have already been logged */
@@ -585,11 +725,9 @@ cli_clone_follow_wait_subprocess(const char *name, pid_t pid)
 					  returnCode);
 		}
 
-		success = success || returnCode == 0;
-
 		/* avoid busy looping, wait for 150ms before checking again */
 		pg_usleep(150 * 1000);
 	}
 
-	return true;
+	return returnCode == 0;
 }
