@@ -28,11 +28,14 @@ typedef struct CopyDBOptions
 
 	int tableJobs;
 	int indexJobs;
+	uint64_t splitTablesLargerThan;
+	char splitTablesLargerThanPretty[NAMEDATALEN];
 
 	RestoreOptions restoreOptions;
 
 	bool roles;
 	bool skipLargeObjects;
+	bool skipExtensions;
 
 	bool restart;
 	bool resume;
@@ -66,4 +69,9 @@ bool cli_copydb_is_consistent(CopyDBOptions *options);
 
 int cli_copy_db_getopts(int argc, char **argv);
 
+bool cli_parse_bytes_pretty(const char *byteString,
+							uint64_t *bytes,
+							char *bytesPretty,
+							size_t bytesPrettySize);
+
 #endif  /* CLI_COMMON_H */
