@@ -13,12 +13,12 @@
 #include "commandline.h"
 #include "copydb.h"
 #include "env_utils.h"
+#include "ld_stream.h"
 #include "log.h"
 #include "pgcmd.h"
 #include "pgsql.h"
 #include "schema.h"
 #include "signals.h"
-#include "stream.h"
 #include "string_utils.h"
 
 static int cli_create_snapshot_getopts(int argc, char **argv);
@@ -128,6 +128,8 @@ cli_create_snapshot_getopts(int argc, char **argv)
 		{ "dir", required_argument, NULL, 'D' },
 		{ "version", no_argument, NULL, 'V' },
 		{ "verbose", no_argument, NULL, 'v' },
+		{ "debug", no_argument, NULL, 'd' },
+		{ "trace", no_argument, NULL, 'z' },
 		{ "quiet", no_argument, NULL, 'q' },
 		{ "help", no_argument, NULL, 'h' },
 		{ NULL, 0, NULL, 0 }
@@ -142,7 +144,7 @@ cli_create_snapshot_getopts(int argc, char **argv)
 		exit(EXIT_CODE_BAD_ARGS);
 	}
 
-	while ((c = getopt_long(argc, argv, "S:T:D:Vvqh",
+	while ((c = getopt_long(argc, argv, "S:T:D:Vvdzqh",
 							long_options, &option_index)) != -1)
 	{
 		switch (c)
@@ -181,7 +183,7 @@ cli_create_snapshot_getopts(int argc, char **argv)
 				{
 					case 1:
 					{
-						log_set_level(LOG_INFO);
+						log_set_level(LOG_NOTICE);
 						break;
 					}
 
@@ -200,6 +202,20 @@ cli_create_snapshot_getopts(int argc, char **argv)
 				break;
 			}
 
+			case 'd':
+			{
+				verboseCount = 2;
+				log_set_level(LOG_DEBUG);
+				break;
+			}
+
+			case 'z':
+			{
+				verboseCount = 3;
+				log_set_level(LOG_TRACE);
+				break;
+			}
+
 			case 'q':
 			{
 				log_set_level(LOG_ERROR);
@@ -278,11 +294,14 @@ cli_create_snapshot(int argc, char **argv)
 						   createSNoptions.target_pguri,
 						   1,   /* tableJobs */
 						   1,   /* indexJobs */
+						   0,   /* skip threshold */
+						   "",  /* skip threshold pretty printed */
 						   DATA_SECTION_ALL,
 						   createSNoptions.snapshot,
 						   restoreOptions,
 						   false, /* roles */
 						   false, /* skipLargeObjects */
+						   false, /* skipExtensions */
 						   createSNoptions.restart,
 						   createSNoptions.resume,
 						   !createSNoptions.notConsistent))
@@ -337,6 +356,8 @@ cli_create_slot_getopts(int argc, char **argv)
 		{ "snapshot", required_argument, NULL, 'N' },
 		{ "version", no_argument, NULL, 'V' },
 		{ "verbose", no_argument, NULL, 'v' },
+		{ "debug", no_argument, NULL, 'd' },
+		{ "trace", no_argument, NULL, 'z' },
 		{ "quiet", no_argument, NULL, 'q' },
 		{ "help", no_argument, NULL, 'h' },
 		{ NULL, 0, NULL, 0 }
@@ -354,7 +375,7 @@ cli_create_slot_getopts(int argc, char **argv)
 	/* pretend that --resume was used */
 	options.resume = true;
 
-	while ((c = getopt_long(argc, argv, "S:T:D:Vvqh",
+	while ((c = getopt_long(argc, argv, "S:T:D:Vvdzqh",
 							long_options, &option_index)) != -1)
 	{
 		switch (c)
@@ -407,7 +428,7 @@ cli_create_slot_getopts(int argc, char **argv)
 				{
 					case 1:
 					{
-						log_set_level(LOG_INFO);
+						log_set_level(LOG_NOTICE);
 						break;
 					}
 
@@ -426,6 +447,20 @@ cli_create_slot_getopts(int argc, char **argv)
 				break;
 			}
 
+			case 'd':
+			{
+				verboseCount = 2;
+				log_set_level(LOG_DEBUG);
+				break;
+			}
+
+			case 'z':
+			{
+				verboseCount = 3;
+				log_set_level(LOG_TRACE);
+				break;
+			}
+
 			case 'q':
 			{
 				log_set_level(LOG_ERROR);
@@ -518,11 +553,14 @@ cli_create_slot(int argc, char **argv)
 						   createSlotOptions.target_pguri,
 						   1,   /* tableJobs */
 						   1,   /* indexJobs */
+						   0,   /* skip threshold */
+						   "",  /* skip threshold pretty printed */
 						   DATA_SECTION_ALL,
 						   createSlotOptions.snapshot,
 						   restoreOptions,
 						   false, /* roles */
 						   false, /* skipLargeObjects */
+						   false, /* skipExtensions */
 						   createSlotOptions.restart,
 						   createSlotOptions.resume,
 						   !createSlotOptions.notConsistent))
@@ -570,11 +608,14 @@ cli_drop_slot(int argc, char **argv)
 						   createSlotOptions.target_pguri,
 						   1,   /* tableJobs */
 						   1,   /* indexJobs */
+						   0,   /* skip threshold */
+						   "",  /* skip threshold pretty printed */
 						   DATA_SECTION_ALL,
 						   createSlotOptions.snapshot,
 						   restoreOptions,
 						   false, /* roles */
 						   false, /* skipLargeObjects */
+						   false, /* skipExtensions */
 						   createSlotOptions.restart,
 						   createSlotOptions.resume,
 						   !createSlotOptions.notConsistent))
@@ -618,6 +659,8 @@ cli_create_origin_getopts(int argc, char **argv)
 		{ "startpos", required_argument, NULL, 's' },
 		{ "version", no_argument, NULL, 'V' },
 		{ "verbose", no_argument, NULL, 'v' },
+		{ "debug", no_argument, NULL, 'd' },
+		{ "trace", no_argument, NULL, 'z' },
 		{ "quiet", no_argument, NULL, 'q' },
 		{ "help", no_argument, NULL, 'h' },
 		{ NULL, 0, NULL, 0 }
@@ -635,7 +678,7 @@ cli_create_origin_getopts(int argc, char **argv)
 	/* pretend that --resume was used */
 	options.resume = true;
 
-	while ((c = getopt_long(argc, argv, "T:D:o:s:Vvqh",
+	while ((c = getopt_long(argc, argv, "T:D:o:s:Vvdzqh",
 							long_options, &option_index)) != -1)
 	{
 		switch (c)
@@ -694,7 +737,7 @@ cli_create_origin_getopts(int argc, char **argv)
 				{
 					case 1:
 					{
-						log_set_level(LOG_INFO);
+						log_set_level(LOG_NOTICE);
 						break;
 					}
 
@@ -713,6 +756,20 @@ cli_create_origin_getopts(int argc, char **argv)
 				break;
 			}
 
+			case 'd':
+			{
+				verboseCount = 2;
+				log_set_level(LOG_DEBUG);
+				break;
+			}
+
+			case 'z':
+			{
+				verboseCount = 3;
+				log_set_level(LOG_TRACE);
+				break;
+			}
+
 			case 'q':
 			{
 				log_set_level(LOG_ERROR);
@@ -809,11 +866,14 @@ cli_create_origin(int argc, char **argv)
 						   createOriginOptions.target_pguri,
 						   1,   /* tableJobs */
 						   1,   /* indexJobs */
+						   0,   /* skip threshold */
+						   "",  /* skip threshold pretty printed */
 						   DATA_SECTION_ALL,
 						   createOriginOptions.snapshot,
 						   restoreOptions,
 						   false, /* roles */
 						   false, /* skipLargeObjects */
+						   false, /* skipExtensions */
 						   createOriginOptions.restart,
 						   createOriginOptions.resume,
 						   !createOriginOptions.notConsistent))
@@ -861,11 +921,14 @@ cli_drop_origin(int argc, char **argv)
 						   createOriginOptions.target_pguri,
 						   1,   /* tableJobs */
 						   1,   /* indexJobs */
+						   0,   /* skip threshold */
+						   "",  /* skip threshold pretty printed */
 						   DATA_SECTION_ALL,
 						   createOriginOptions.snapshot,
 						   restoreOptions,
 						   false, /* roles */
 						   false, /* skipLargeObjects */
+						   false, /* skipExtensions */
 						   createOriginOptions.restart,
 						   createOriginOptions.resume,
 						   !createOriginOptions.notConsistent))
