@@ -17,12 +17,12 @@
 #include "commandline.h"
 #include "copydb.h"
 #include "env_utils.h"
+#include "ld_stream.h"
 #include "log.h"
 #include "pgcmd.h"
 #include "pgsql.h"
 #include "schema.h"
 #include "signals.h"
-#include "stream.h"
 #include "string_utils.h"
 
 CopyDBOptions streamDBoptions = { 0 };
@@ -43,7 +43,7 @@ static CommandLine stream_setup_command =
 	make_command(
 		"setup",
 		"Setup source and target systems for logical decoding",
-		" --source ... --target ... --dir ...",
+		"",
 		"  --source         Postgres URI to the source database\n"
 		"  --target         Postgres URI to the target database\n"
 		"  --dir            Work directory to use\n"
@@ -60,7 +60,7 @@ static CommandLine stream_cleanup_command =
 	make_command(
 		"cleanup",
 		"cleanup source and target systems for logical decoding",
-		" --source ... --target ... --dir ...",
+		"",
 		"  --source         Postgres URI to the source database\n"
 		"  --target         Postgres URI to the target database\n"
 		"  --dir            Work directory to use\n"
@@ -77,7 +77,7 @@ static CommandLine stream_prefetch_command =
 	make_command(
 		"prefetch",
 		"Stream JSON changes from the source database and transform them to SQL",
-		" --source ... ",
+		"",
 		"  --source         Postgres URI to the source database\n"
 		"  --dir            Work directory to use\n"
 		"  --restart        Allow restarting when temp files exist already\n"
@@ -92,7 +92,7 @@ static CommandLine stream_catchup_command =
 	make_command(
 		"catchup",
 		"Apply prefetched changes from SQL files to the target database",
-		" --source ... --target ...",
+		"",
 		"  --source         Postgres URI to the source database\n"
 		"  --target         Postgres URI to the target database\n"
 		"  --dir            Work directory to use\n"
@@ -109,7 +109,7 @@ static CommandLine stream_receive_command =
 	make_command(
 		"receive",
 		"Stream changes from the source database",
-		" --source ... ",
+		"",
 		"  --source         Postgres URI to the source database\n"
 		"  --dir            Work directory to use\n"
 		"  --restart        Allow restarting when temp files exist already\n"
@@ -124,7 +124,7 @@ static CommandLine stream_transform_command =
 	make_command(
 		"transform",
 		"Transform changes from the source database into SQL commands",
-		" [ --source ... ] <json filename> <sql filename> ",
+		" <json filename> <sql filename> ",
 		"  --source         Postgres URI to the source database\n"
 		"  --dir            Work directory to use\n"
 		"  --restart        Allow restarting when temp files exist already\n"
@@ -137,7 +137,7 @@ static CommandLine stream_apply_command =
 	make_command(
 		"apply",
 		"Apply changes from the source database into the target database",
-		" --target ... <sql filename> ",
+		" <sql filename> ",
 		"  --target         Postgres URI to the target database\n"
 		"  --dir            Work directory to use\n"
 		"  --restart        Allow restarting when temp files exist already\n"
@@ -191,6 +191,8 @@ cli_stream_getopts(int argc, char **argv)
 		{ "not-consistent", no_argument, NULL, 'C' },
 		{ "version", no_argument, NULL, 'V' },
 		{ "verbose", no_argument, NULL, 'v' },
+		{ "debug", no_argument, NULL, 'd' },
+		{ "trace", no_argument, NULL, 'z' },
 		{ "quiet", no_argument, NULL, 'q' },
 		{ "help", no_argument, NULL, 'h' },
 		{ NULL, 0, NULL, 0 }
@@ -205,7 +207,7 @@ cli_stream_getopts(int argc, char **argv)
 		exit(EXIT_CODE_BAD_ARGS);
 	}
 
-	while ((c = getopt_long(argc, argv, "S:T:j:s:o:t:PVvqh",
+	while ((c = getopt_long(argc, argv, "S:T:j:s:o:t:PVvdzqh",
 							long_options, &option_index)) != -1)
 	{
 		switch (c)
@@ -323,7 +325,7 @@ cli_stream_getopts(int argc, char **argv)
 				{
 					case 1:
 					{
-						log_set_level(LOG_INFO);
+						log_set_level(LOG_NOTICE);
 						break;
 					}
 
@@ -342,6 +344,20 @@ cli_stream_getopts(int argc, char **argv)
 				break;
 			}
 
+			case 'd':
+			{
+				verboseCount = 2;
+				log_set_level(LOG_DEBUG);
+				break;
+			}
+
+			case 'z':
+			{
+				verboseCount = 3;
+				log_set_level(LOG_TRACE);
+				break;
+			}
+
 			case 'q':
 			{
 				log_set_level(LOG_ERROR);
@@ -417,6 +433,12 @@ cli_stream_getopts(int argc, char **argv)
 static void
 cli_stream_receive(int argc, char **argv)
 {
+	if (argc > 0)
+	{
+		commandline_help(stderr);
+		exit(EXIT_CODE_BAD_ARGS);
+	}
+
 	(void) stream_start_in_mode(STREAM_MODE_RECEIVE);
 }
 
@@ -428,6 +450,12 @@ cli_stream_receive(int argc, char **argv)
 static void
 cli_stream_prefetch(int argc, char **argv)
 {
+	if (argc > 0)
+	{
+		commandline_help(stderr);
+		exit(EXIT_CODE_BAD_ARGS);
+	}
+
 	(void) stream_start_in_mode(STREAM_MODE_PREFETCH);
 }
 
@@ -445,6 +473,12 @@ cli_stream_setup(int argc, char **argv)
 {
 	CopyDataSpec copySpecs = { 0 };
 
+	if (argc > 0)
+	{
+		commandline_help(stderr);
+		exit(EXIT_CODE_BAD_ARGS);
+	}
+
 	(void) find_pg_commands(&(copySpecs.pgPaths));
 
 	bool auxilliary = false;
@@ -466,11 +500,14 @@ cli_stream_setup(int argc, char **argv)
 						   streamDBoptions.target_pguri,
 						   1,   /* tableJobs */
 						   1,   /* indexJobs */
+						   0,   /* skip threshold */
+						   "",  /* skip threshold pretty printed */
 						   DATA_SECTION_ALL,
 						   streamDBoptions.snapshot,
 						   restoreOptions,
 						   false, /* roles */
 						   false, /* skipLargeObjects */
+						   false, /* skipExtensions */
 						   streamDBoptions.restart,
 						   streamDBoptions.resume,
 						   !streamDBoptions.notConsistent))
@@ -498,6 +535,12 @@ cli_stream_cleanup(int argc, char **argv)
 {
 	CopyDataSpec copySpecs = { 0 };
 
+	if (argc > 0)
+	{
+		commandline_help(stderr);
+		exit(EXIT_CODE_BAD_ARGS);
+	}
+
 	(void) find_pg_commands(&(copySpecs.pgPaths));
 
 	bool resume = true;         /* pretend --resume has been used */
@@ -521,11 +564,14 @@ cli_stream_cleanup(int argc, char **argv)
 						   streamDBoptions.target_pguri,
 						   1,   /* tableJobs */
 						   1,   /* indexJobs */
+						   0,   /* skip threshold */
+						   "",  /* skip threshold pretty printed */
 						   DATA_SECTION_ALL,
 						   streamDBoptions.snapshot,
 						   restoreOptions,
 						   false, /* roles */
 						   false, /* skipLargeObjects */
+						   false, /* skipExtensions */
 						   restart,
 						   resume,
 						   !streamDBoptions.notConsistent))
@@ -553,6 +599,12 @@ cli_stream_catchup(int argc, char **argv)
 {
 	CopyDataSpec copySpecs = { 0 };
 
+	if (argc > 0)
+	{
+		commandline_help(stderr);
+		exit(EXIT_CODE_BAD_ARGS);
+	}
+
 	(void) find_pg_commands(&(copySpecs.pgPaths));
 
 	bool auxilliary = false;
@@ -574,11 +626,14 @@ cli_stream_catchup(int argc, char **argv)
 						   streamDBoptions.target_pguri,
 						   1,   /* tableJobs */
 						   1,   /* indexJobs */
+						   0,   /* skip threshold */
+						   "",  /* skip threshold pretty printed */
 						   DATA_SECTION_ALL,
 						   streamDBoptions.snapshot,
 						   restoreOptions,
 						   false, /* roles */
 						   false, /* skipLargeObjects */
+						   false, /* skipExtensions */
 						   streamDBoptions.restart,
 						   streamDBoptions.resume,
 						   !streamDBoptions.notConsistent))
@@ -678,11 +733,14 @@ cli_stream_apply(int argc, char **argv)
 						   streamDBoptions.target_pguri,
 						   1,   /* tableJobs */
 						   1,   /* indexJobs */
+						   0,   /* skip threshold */
+						   "",  /* skip threshold pretty printed */
 						   DATA_SECTION_ALL,
 						   streamDBoptions.snapshot,
 						   restoreOptions,
 						   false, /* roles */
 						   false, /* skipLargeObjects */
+						   false, /* skipExtensions */
 						   streamDBoptions.restart,
 						   streamDBoptions.resume,
 						   !streamDBoptions.notConsistent))
@@ -754,11 +812,14 @@ stream_start_in_mode(LogicalStreamMode mode)
 						   streamDBoptions.target_pguri,
 						   1,   /* tableJobs */
 						   1,   /* indexJobs */
+						   0,   /* skip threshold */
+						   "",  /* skip threshold pretty printed */
 						   DATA_SECTION_ALL,
 						   streamDBoptions.snapshot,
 						   restoreOptions,
 						   false, /* roles */
 						   false, /* skipLargeObjects */
+						   false, /* skipExtensions */
 						   streamDBoptions.restart,
 						   streamDBoptions.resume,
 						   !streamDBoptions.notConsistent))
