@@ -9,12 +9,14 @@
 #include <sys/wait.h>
 #include <unistd.h>
 
+#include "parson.h"
+
 #include "cli_common.h"
 #include "copydb.h"
 #include "env_utils.h"
 #include "lock_utils.h"
 #include "log.h"
-#include "parsing.h"
+#include "parsing_utils.h"
 #include "pidfile.h"
 #include "schema.h"
 #include "signals.h"
@@ -22,11 +24,20 @@
 #include "summary.h"
 
 #define COMMON_GUC_SETTINGS \
-	{ "client_encoding", "'UTF-8'" },
+	{ "client_encoding", "'UTF-8'" }, \
+	{ "tcp_keepalives_idle", "'60s'" },
+
+
+/* Postgres 9.5 does not have idle_in_transaction_session_timeout */
+GUC srcSettings95[] = {
+	COMMON_GUC_SETTINGS
+	{ NULL, NULL },
+};
+
 
 GUC srcSettings[] = {
 	COMMON_GUC_SETTINGS
-	{ "tcp_keepalives_idle", "'60s'" },
+	{ "idle_in_transaction_session_timeout", "0" },
 	{ NULL, NULL },
 };
 
@@ -73,7 +84,7 @@ copydb_init_workdir(CopyDataSpec *copySpecs,
 		return false;
 	}
 
-	log_info("Using work dir \"%s\"", cfPaths->topdir);
+	log_notice("Using work dir \"%s\"", cfPaths->topdir);
 
 	/* check to see if there is already another pgcopydb running */
 	if (directory_exists(cfPaths->topdir))
@@ -101,62 +112,64 @@ copydb_init_workdir(CopyDataSpec *copySpecs,
 
 	bool removeDir = false;
 
-	if (!copydb_inspect_workdir(cfPaths, dirState))
+	if (restart)
 	{
-		/* errors have already been logged */
-		return false;
+		removeDir = true;
 	}
-
-	if (dirState->directoryExists)
+	else
 	{
-		if (restart)
+		if (!copydb_inspect_workdir(cfPaths, dirState))
 		{
-			removeDir = true;
-		}
-
-		/* if we did nothing yet, just act as if --resume was used */
-		else if (!dirState->schemaDumpIsDone)
-		{
-			log_debug("schema dump has not been done yet, just continue");
-		}
-
-		/* if --resume has been used, we just continue */
-		else if (resume)
-		{
-			/* no-op */
-			(void) 0;
-		}
-		else if (dirState->allDone)
-		{
-			log_fatal("Please use --restart to allow for removing files "
-					  "that belong to a completed previous run.");
-			return false;
-		}
-		else if (!resume)
-		{
-			log_fatal("Please use --resume --not-consistent to allow "
-					  "for resuming from the previous run, "
-					  "which failed before completion.");
+			/* errors have already been logged */
 			return false;
 		}
 
-		/*
-		 * Here we should have restart true or resume true or we didn't even do
-		 * the schema dump on the previous run.
-		 */
+		if (dirState->directoryExists)
+		{
+			/* if we did nothing yet, just act as if --resume was used */
+			if (!dirState->schemaDumpIsDone)
+			{
+				log_notice("Schema dump has not been done yet, just continue");
+			}
+
+			/* if --resume has been used, we just continue */
+			else if (resume)
+			{
+				/* no-op */
+				(void) 0;
+			}
+			else if (dirState->allDone)
+			{
+				log_fatal("Please use --restart to allow for removing files "
+						  "that belong to a completed previous run.");
+				return false;
+			}
+			else if (!resume)
+			{
+				log_fatal("Please use --resume --not-consistent to allow "
+						  "for resuming from the previous run, "
+						  "which failed before completion.");
+				return false;
+			}
+
+			/*
+			 * Here we should have restart true or resume true or we didn't even do
+			 * the schema dump on the previous run.
+			 */
+		}
 	}
 
 	/* warn about trashing data from a previous run */
 	if (removeDir && !restart)
 	{
-		log_info("Inspection of \"%s\" shows that it is safe "
-				 "to remove it and continue",
-				 cfPaths->topdir);
+		log_notice("Inspection of \"%s\" shows that it is safe "
+				   "to remove it and continue",
+				   cfPaths->topdir);
 	}
 
 	if (removeDir)
 	{
-		log_info("Removing directory \"%s\"", cfPaths->topdir);
+		log_notice("Removing directory \"%s\"", cfPaths->topdir);
 	}
 
 	/* make sure the directory exists, possibly making it empty */
@@ -259,7 +272,7 @@ copydb_inspect_workdir(CopyFilePaths *cfPaths, DirectoryState *dirState)
 		dirState->blobsCopyIsDone;
 
 	/* let's be verbose about our inspection results */
-	log_info("Work directory \"%s\" already exists", cfPaths->topdir);
+	log_notice("Work directory \"%s\" already exists", cfPaths->topdir);
 
 	if (dirState->allDone)
 	{
@@ -355,6 +368,9 @@ copydb_prepare_filepaths(CopyFilePaths *cfPaths, const char *dir, bool auxilliar
 	sformat(cfPaths->tbldir, MAXPGPATH, "%s/run/tables", cfPaths->topdir);
 	sformat(cfPaths->idxdir, MAXPGPATH, "%s/run/indexes", cfPaths->topdir);
 
+	/* prepare also the name of the schema file (JSON) */
+	sformat(cfPaths->schemafile, MAXPGPATH, "%s/schema.json", cfPaths->topdir);
+
 	/* now prepare the done files */
 	struct pair
 	{
@@ -454,6 +470,9 @@ copydb_prepare_dump_paths(CopyFilePaths *cfPaths, DumpPaths *dumpPaths)
 	sformat(dumpPaths->rolesFilename, MAXPGPATH, "%s/%s",
 			cfPaths->schemadir, "roles.sql");
 
+	sformat(dumpPaths->extnspFilename, MAXPGPATH, "%s/%s",
+			cfPaths->schemadir, "extnamespaces.dump");
+
 	sformat(dumpPaths->preFilename, MAXPGPATH, "%s/%s",
 			cfPaths->schemadir, "pre.dump");
 
@@ -489,7 +508,10 @@ copydb_rmdir_or_mkdir(const char *dir, bool removeDir)
 	}
 	else
 	{
-		log_debug("mkdir -p \"%s\"", dir);
+		if (!directory_exists(dir))
+		{
+			log_debug("mkdir -p \"%s\"", dir);
+		}
 
 		if (pg_mkdir_p((char *) dir, 0700) == -1)
 		{
@@ -514,11 +536,14 @@ copydb_init_specs(CopyDataSpec *specs,
 				  char *target_pguri,
 				  int tableJobs,
 				  int indexJobs,
+				  uint64_t splitTablesLargerThan,
+				  char *splitTablesLargerThanPretty,
 				  CopyDataSection section,
 				  char *snapshot,
 				  RestoreOptions restoreOptions,
 				  bool roles,
 				  bool skipLargeObjects,
+				  bool skipExtensions,
 				  bool restart,
 				  bool resume,
 				  bool consistent)
@@ -542,6 +567,7 @@ copydb_init_specs(CopyDataSpec *specs,
 		.restoreOptions = restoreOptions,
 		.roles = roles,
 		.skipLargeObjects = skipLargeObjects,
+		.skipExtensions = skipExtensions,
 
 		.restart = restart,
 		.resume = resume,
@@ -549,7 +575,25 @@ copydb_init_specs(CopyDataSpec *specs,
 
 		.tableJobs = tableJobs,
 		.indexJobs = indexJobs,
-		.indexSemaphore = { 0 }
+
+		/* at the moment we don't have --vacuumJobs separately */
+		.vacuumJobs = tableJobs,
+
+		.splitTablesLargerThan = splitTablesLargerThan,
+
+		.tableSemaphore = { 0 },
+		.indexSemaphore = { 0 },
+
+		.vacuumQueue = { 0 },
+		.indexQueue = { 0 },
+
+		.extensionArray = { 0, NULL },
+		.sourceTableArray = { 0, NULL },
+		.sourceIndexArray = { 0, NULL },
+		.sequenceArray = { 0, NULL },
+		.tableSpecsArray = { 0, NULL },
+
+		.sourceTableHashByOid = NULL
 	};
 
 	/* initialize the connection strings */
@@ -571,6 +615,10 @@ copydb_init_specs(CopyDataSpec *specs,
 				sizeof(tmpCopySpecs.sourceSnapshot.snapshot));
 	}
 
+	strlcpy(tmpCopySpecs.splitTablesLargerThanPretty,
+			splitTablesLargerThanPretty,
+			sizeof(tmpCopySpecs.splitTablesLargerThanPretty));
+
 	/* copy the structure as a whole memory area to the target place */
 	*specs = tmpCopySpecs;
 
@@ -592,17 +640,35 @@ copydb_init_specs(CopyDataSpec *specs,
 		return false;
 	}
 
-	/* create the index semaphore (allow jobs to start) */
-	specs->indexSemaphore.initValue = indexJobs;
+	/* create the index semaphore (critical section, one at a time please) */
+	specs->indexSemaphore.initValue = 1;
 
 	if (!semaphore_create(&(specs->indexSemaphore)))
 	{
 		log_error("Failed to create the index concurrency semaphore "
-				  "to orchestrate up to %d CREATE INDEX jobs at the same time",
+				  "to orchestrate %d CREATE INDEX jobs",
 				  indexJobs);
 		return false;
 	}
 
+	if (specs->section == DATA_SECTION_ALL ||
+		specs->section == DATA_SECTION_TABLE_DATA)
+	{
+		/* create the VACUUM process queue */
+		if (!queue_create(&(specs->vacuumQueue)))
+		{
+			log_error("Failed to create the VACUUM process queue");
+			return false;
+		}
+
+		/* create the CREATE INDEX process queue */
+		if (!queue_create(&(specs->indexQueue)))
+		{
+			log_error("Failed to create the INDEX process queue");
+			return false;
+		}
+	}
+
 	/* we only respect the --skip-blobs option in pgcopydb copy-db command */
 	if (specs->section != DATA_SECTION_ALL)
 	{
@@ -621,7 +687,8 @@ copydb_init_specs(CopyDataSpec *specs,
 bool
 copydb_init_table_specs(CopyTableDataSpec *tableSpecs,
 						CopyDataSpec *specs,
-						SourceTable *source)
+						SourceTable *source,
+						int partNumber)
 {
 	/* fill-in a structure with the help of the C compiler */
 	CopyTableDataSpec tmpTableSpecs = {
@@ -640,12 +707,13 @@ copydb_init_table_specs(CopyTableDataSpec *tableSpecs,
 		.section = specs->section,
 		.resume = specs->resume,
 
-		.sourceTable = { 0 },
+		.sourceTable = source,
 		.indexArray = NULL,
 		.summary = NULL,
 
 		.tableJobs = specs->tableJobs,
 		.indexJobs = specs->indexJobs,
+
 		.indexSemaphore = &(specs->indexSemaphore)
 	};
 
@@ -660,191 +728,91 @@ copydb_init_table_specs(CopyTableDataSpec *tableSpecs,
 		return false;
 	}
 
-	/* copy the SourceTable into our memory area */
-	tmpTableSpecs.sourceTable = *source;
-
 	/* copy the structure as a whole memory area to the target place */
 	*tableSpecs = tmpTableSpecs;
 
 	/* compute the table fully qualified name */
 	sformat(tableSpecs->qname, sizeof(tableSpecs->qname),
 			"\"%s\".\"%s\"",
-			tableSpecs->sourceTable.nspname,
-			tableSpecs->sourceTable.relname);
+			tableSpecs->sourceTable->nspname,
+			tableSpecs->sourceTable->relname);
 
-
-	/* now compute the table-specific paths we are using in copydb */
-	sformat(tableSpecs->tablePaths.lockFile, MAXPGPATH, "%s/%d",
-			tableSpecs->cfPaths->rundir,
-			source->oid);
-
-	sformat(tableSpecs->tablePaths.doneFile, MAXPGPATH, "%s/%d.done",
-			tableSpecs->cfPaths->tbldir,
-			source->oid);
-
-	sformat(tableSpecs->tablePaths.idxListFile, MAXPGPATH, "%s/%u.idx",
-			tableSpecs->cfPaths->tbldir,
-			source->oid);
-
-	return true;
-}
-
-
-/*
- * copydb_copy_snapshot initializes a new TransactionSnapshot from another
- * snapshot that's been exported already, copying the connection string and the
- * snapshot identifier.
- */
-bool
-copydb_copy_snapshot(CopyDataSpec *specs, TransactionSnapshot *snapshot)
-{
-	PGSQL pgsql = { 0 };
-	TransactionSnapshot *source = &(specs->sourceSnapshot);
-
-	/* copy our source snapshot data into the new snapshot instance */
-	snapshot->pgsql = pgsql;
-	snapshot->connectionType = source->connectionType;
-
-	strlcpy(snapshot->pguri, source->pguri, sizeof(snapshot->pguri));
-	strlcpy(snapshot->snapshot, source->snapshot, sizeof(snapshot->snapshot));
-
-	return true;
-}
-
-
-/*
- * copydb_open_snapshot opens a snapshot on the given connection.
- *
- * This is needed in the main process, so that COPY processes can then re-use
- * the snapshot, and thus we get a consistent view of the database all along.
- */
-bool
-copydb_export_snapshot(TransactionSnapshot *snapshot)
-{
-	PGSQL *pgsql = &(snapshot->pgsql);
-
-	log_debug("copydb_export_snapshot");
-
-	if (!pgsql_init(pgsql, snapshot->pguri, snapshot->connectionType))
+	/* This CopyTableDataSpec might be for a partial COPY */
+	if (source->partsArray.count >= 1)
 	{
-		/* errors have already been logged */
-		return false;
-	}
+		CopyTableDataPartSpec part = {
+			.partNumber = partNumber,
+			.partCount = source->partsArray.array[partNumber].partCount,
+			.min = source->partsArray.array[partNumber].min,
+			.max = source->partsArray.array[partNumber].max
+		};
 
-	if (!pgsql_begin(pgsql))
-	{
-		/* errors have already been logged */
-		return false;
-	}
+		tableSpecs->part = part;
 
-	/*
-	 * As Postgres docs for SET TRANSACTION SNAPSHOT say:
-	 *
-	 * Furthermore, the transaction must already be set to SERIALIZABLE or
-	 * REPEATABLE READ isolation level (otherwise, the snapshot would be
-	 * discarded immediately, since READ COMMITTED mode takes a new snapshot
-	 * for each command).
-	 *
-	 * When --filters are used, pgcopydb creates TEMP tables on the source
-	 * database to then implement the filtering as JOINs with the Postgres
-	 * catalogs. And even TEMP tables need read-write transaction.
-	 */
-	IsolationLevel level = ISOLATION_SERIALIZABLE;
-	bool readOnly = false;
-	bool deferrable = true;
+		strlcpy(tableSpecs->part.partKey, source->partKey, NAMEDATALEN);
 
-	if (!pgsql_set_transaction(pgsql, level, readOnly, deferrable))
-	{
-		/* errors have already been logged */
-		(void) pgsql_finish(pgsql);
-		return false;
-	}
-
-	if (!pgsql_export_snapshot(pgsql,
-							   snapshot->snapshot,
-							   sizeof(snapshot->snapshot)))
-	{
-		/* errors have already been logged */
-		(void) pgsql_finish(pgsql);
-		return false;
-	}
-
-	snapshot->state = SNAPSHOT_STATE_EXPORTED;
-
-	log_info("Exported snapshot \"%s\" from the source database",
-			 snapshot->snapshot);
-
-	return true;
-}
-
-
-/*
- * copydb_set_snapshot opens a transaction and set it to re-use an existing
- * snapshot.
- */
-bool
-copydb_set_snapshot(CopyDataSpec *copySpecs)
-{
-	TransactionSnapshot *snapshot = &(copySpecs->sourceSnapshot);
-	PGSQL *pgsql = &(snapshot->pgsql);
-
-	if (!pgsql_init(pgsql, snapshot->pguri, snapshot->connectionType))
-	{
-		/* errors have already been logged */
-		return false;
-	}
-
-	if (!pgsql_begin(pgsql))
-	{
-		/* errors have already been logged */
-		return false;
-	}
-
-	if (copySpecs->consistent)
-	{
 		/*
-		 * As Postgres docs for SET TRANSACTION SNAPSHOT say:
+		 * Prepare the COPY command.
 		 *
-		 * Furthermore, the transaction must already be set to SERIALIZABLE or
-		 * REPEATABLE READ isolation level (otherwise, the snapshot would be
-		 * discarded immediately, since READ COMMITTED mode takes a new
-		 * snapshot for each command).
-		 *
-		 * When --filters are used, pgcopydb creates TEMP tables on the source
-		 * database to then implement the filtering as JOINs with the Postgres
-		 * catalogs. And even TEMP tables need read-write transaction.
+		 * The way schema_list_partitions prepares the boundaries is non
+		 * overlapping, so we can use the BETWEEN operator to select our source
+		 * rows in the COPY sub-query.
 		 */
-		IsolationLevel level = ISOLATION_REPEATABLE_READ;
-		bool readOnly = false;
-		bool deferrable = true;
+		sformat(tableSpecs->part.copyQuery, sizeof(tableSpecs->part.copyQuery),
+				"(SELECT * FROM %s"
+				" WHERE \"%s\" BETWEEN %lld AND %lld)",
+				tableSpecs->qname,
+				tableSpecs->part.partKey,
+				(long long) tableSpecs->part.min,
+				(long long) tableSpecs->part.max);
 
-		if (!pgsql_set_transaction(pgsql, level, readOnly, deferrable))
+		/* now compute the table-specific paths we are using in copydb */
+		if (!copydb_init_tablepaths_for_part(tableSpecs->cfPaths,
+											 &(tableSpecs->tablePaths),
+											 tableSpecs->sourceTable->oid,
+											 partNumber))
 		{
-			/* errors have already been logged */
-			(void) pgsql_finish(pgsql);
+			log_error("Failed to prepare pathnames for partition %d of table %s",
+					  partNumber,
+					  tableSpecs->qname);
 			return false;
 		}
 
-		if (!pgsql_set_snapshot(pgsql, snapshot->snapshot))
-		{
-			/* errors have already been logged */
-			(void) pgsql_finish(pgsql);
-			return false;
-		}
+		/* used only by one process, the one finishing a partial COPY last */
+		sformat(tableSpecs->tablePaths.idxListFile, MAXPGPATH, "%s/%u.idx",
+				tableSpecs->cfPaths->tbldir,
+				source->oid);
 
-		copySpecs->sourceSnapshot.state = SNAPSHOT_STATE_SET;
+		/*
+		 * And now the truncateLockFile and truncateDoneFile, which are used to
+		 * provide a critical section to the same-table concurrent processes.
+		 */
+		sformat(tableSpecs->tablePaths.truncateDoneFile, MAXPGPATH,
+				"%s/%u.truncate",
+				tableSpecs->cfPaths->tbldir,
+				source->oid);
 	}
 	else
 	{
-		copySpecs->sourceSnapshot.state = SNAPSHOT_STATE_NOT_CONSISTENT;
-	}
+		/* No partition found, so this should be a full table COPY */
+		if (partNumber > 0)
+		{
+			log_error("BUG: copydb_init_table_specs partNumber is %d and "
+					  "source table partArray.count is %d",
+					  partNumber,
+					  source->partsArray.count);
+			return false;
+		}
 
-	/* also set our GUC values for the source connection */
-	if (!pgsql_set_gucs(pgsql, srcSettings))
-	{
-		log_fatal("Failed to set our GUC settings on the source connection, "
-				  "see above for details");
-		return false;
+		/* now compute the table-specific paths we are using in copydb */
+		if (!copydb_init_tablepaths(tableSpecs->cfPaths,
+									&(tableSpecs->tablePaths),
+									tableSpecs->sourceTable->oid))
+		{
+			log_error("Failed to prepare pathnames for table %u",
+					  tableSpecs->sourceTable->oid);
+			return false;
+		}
 	}
 
 	return true;
@@ -852,175 +820,50 @@ copydb_set_snapshot(CopyDataSpec *copySpecs)
 
 
 /*
- * copydb_close_snapshot closes the snapshot on Postgres by committing the
- * transaction and finishing the connection.
+ * copydb_init_tablepaths computes the lockFile, doneFile, and idxListFile
+ * pathnames for a given table oid and global cfPaths setup.
  */
 bool
-copydb_close_snapshot(CopyDataSpec *copySpecs)
+copydb_init_tablepaths(CopyFilePaths *cfPaths,
+					   TableFilePaths *tablePaths,
+					   uint32_t oid)
 {
-	TransactionSnapshot *snapshot = &(copySpecs->sourceSnapshot);
-	PGSQL *pgsql = &(snapshot->pgsql);
+	sformat(tablePaths->lockFile, MAXPGPATH, "%s/%d",
+			cfPaths->rundir,
+			oid);
 
-	if (copySpecs->sourceSnapshot.state == SNAPSHOT_STATE_SET ||
-		copySpecs->sourceSnapshot.state == SNAPSHOT_STATE_EXPORTED ||
-		copySpecs->sourceSnapshot.state == SNAPSHOT_STATE_NOT_CONSISTENT)
-	{
-		if (!pgsql_commit(pgsql))
-		{
-			char pguri[MAXCONNINFO] = { 0 };
+	sformat(tablePaths->doneFile, MAXPGPATH, "%s/%d.done",
+			cfPaths->tbldir,
+			oid);
 
-			(void) parse_and_scrub_connection_string(snapshot->pguri, pguri);
-
-			log_fatal("Failed to close snapshot \"%s\" on \"%s\"",
-					  snapshot->snapshot,
-					  pguri);
-			return false;
-		}
-
-		(void) pgsql_finish(pgsql);
-	}
-
-	copySpecs->sourceSnapshot.state = SNAPSHOT_STATE_CLOSED;
+	sformat(tablePaths->idxListFile, MAXPGPATH, "%s/%u.idx",
+			cfPaths->tbldir,
+			oid);
 
 	return true;
 }
 
 
 /*
- * copydb_prepare_snapshot connects to the source database and either export a
- * new Postgres snapshot, or set the transaction's snapshot to the given
- * already exported snapshot (see --snapshot and PGCOPYDB_SNAPSHOT).
+ * copydb_init_tablepaths_for_part computes the lockFile and doneFile pathnames
+ * for a given COPY partition of a table.
  */
 bool
-copydb_prepare_snapshot(CopyDataSpec *copySpecs)
+copydb_init_tablepaths_for_part(CopyFilePaths *cfPaths,
+								TableFilePaths *tablePaths,
+								uint32_t oid,
+								int partNumber)
 {
-	/* when --not-consistent is used, we have nothing to do here */
-	if (!copySpecs->consistent)
-	{
-		copySpecs->sourceSnapshot.state = SNAPSHOT_STATE_SKIPPED;
-		log_debug("copydb_prepare_snapshot: --not-consistent, skipping");
-		return true;
-	}
+	sformat(tablePaths->lockFile, MAXPGPATH, "%s/%d.%d",
+			cfPaths->rundir,
+			oid,
+			partNumber);
 
-	/*
-	 * First, we need to open a snapshot that we're going to re-use in all our
-	 * connections to the source database. When the --snapshot option has been
-	 * used, instead of exporting a new snapshot, we can just re-use it.
-	 */
-	TransactionSnapshot *sourceSnapshot = &(copySpecs->sourceSnapshot);
+	sformat(tablePaths->doneFile, MAXPGPATH, "%s/%d.%d.done",
+			cfPaths->tbldir,
+			oid,
+			partNumber);
 
-	if (IS_EMPTY_STRING_BUFFER(sourceSnapshot->snapshot))
-	{
-		if (!copydb_export_snapshot(sourceSnapshot))
-		{
-			log_fatal("Failed to export a snapshot on \"%s\"",
-					  sourceSnapshot->pguri);
-			return false;
-		}
-	}
-	else
-	{
-		if (!copydb_set_snapshot(copySpecs))
-		{
-			log_fatal("Failed to use given --snapshot \"%s\"",
-					  sourceSnapshot->snapshot);
-			return false;
-		}
-
-		log_info("[SNAPSHOT] Using snapshot \"%s\" on the source database",
-				 sourceSnapshot->snapshot);
-	}
-
-	/* store the snapshot in a file, to support --resume --snapshot ... */
-	if (!write_file(sourceSnapshot->snapshot,
-					strlen(sourceSnapshot->snapshot),
-					copySpecs->cfPaths.snfile))
-	{
-		log_fatal("Failed to create the snapshot file \"%s\"",
-				  copySpecs->cfPaths.snfile);
-		return false;
-	}
-
-	/* also set our GUC values for the source connection */
-	if (!pgsql_set_gucs(&(sourceSnapshot->pgsql), srcSettings))
-	{
-		log_fatal("Failed to set our GUC settings on the source connection, "
-				  "see above for details");
-		return false;
-	}
-
-	return true;
-}
-
-
-/*
- * copydb_start_vacuum_table runs VACUUM ANALYSE on the given table.
- */
-bool
-copydb_start_vacuum_table(CopyTableDataSpec *tableSpecs)
-{
-	if (tableSpecs->section != DATA_SECTION_VACUUM &&
-		tableSpecs->section != DATA_SECTION_ALL)
-	{
-		return true;
-	}
-
-	/*
-	 * Flush stdio channels just before fork, to avoid double-output problems.
-	 */
-	fflush(stdout);
-	fflush(stderr);
-
-	int fpid = fork();
-
-	switch (fpid)
-	{
-		case -1:
-		{
-			log_error("Failed to fork a worker process");
-			return false;
-		}
-
-		case 0:
-		{
-			/* child process runs the command */
-			PGSQL dst = { 0 };
-
-			/* initialize our connection to the target database */
-			if (!pgsql_init(&dst, tableSpecs->target_pguri, PGSQL_CONN_TARGET))
-			{
-				/* errors have already been logged */
-				return false;
-			}
-
-			/* finally, vacuum analyze the table and its indexes */
-			char vacuum[BUFSIZE] = { 0 };
-
-			sformat(vacuum, sizeof(vacuum),
-					"VACUUM ANALYZE %s",
-					tableSpecs->qname);
-
-			log_info("%s;", vacuum);
-
-			if (!pgsql_execute(&dst, vacuum))
-			{
-				/* errors have already been logged */
-				exit(EXIT_CODE_INTERNAL_ERROR);
-			}
-
-			(void) pgsql_finish(&dst);
-
-			exit(EXIT_CODE_QUIT);
-		}
-
-		default:
-		{
-			/* fork succeeded, in parent */
-			break;
-		}
-	}
-
-	/* now we're done, and we want async behavior, do not wait */
 	return true;
 }
 
@@ -1071,6 +914,7 @@ copydb_wait_for_subprocesses()
 				if (errno == ECHILD)
 				{
 					/* no more childrens */
+					log_debug("copydb_wait_for_subprocesses: no more children");
 					return allReturnCodeAreZero;
 				}
 
@@ -1088,69 +932,6 @@ copydb_wait_for_subprocesses()
 				break;
 			}
 
-			default:
-			{
-				int returnCode = WEXITSTATUS(status);
-
-				if (returnCode == 0)
-				{
-					log_debug("Sub-processes %d exited with code %d",
-							  pid, returnCode);
-				}
-				else
-				{
-					allReturnCodeAreZero = false;
-
-					log_error("Sub-processes %d exited with code %d",
-							  pid, returnCode);
-				}
-			}
-		}
-	}
-
-	return allReturnCodeAreZero;
-}
-
-
-/*
- * copydb_collect_finished_subprocesses calls waitpid() to acknowledge finished
- * processes, without waiting for all of them.
- */
-bool
-copydb_collect_finished_subprocesses(bool *allDone)
-{
-	bool allReturnCodeAreZero = true;
-
-	for (;;)
-	{
-		int status;
-
-		/* ignore errors */
-		pid_t pid = waitpid(-1, &status, WNOHANG);
-
-		switch (pid)
-		{
-			case -1:
-			{
-				if (errno == ECHILD)
-				{
-					/* no more childrens */
-					return allReturnCodeAreZero;
-				}
-
-				break;
-			}
-
-			case 0:
-			{
-				/*
-				 * We're using WNOHANG, 0 means there are no stopped or exited
-				 * children.
-				 */
-				*allDone = true;
-				return allReturnCodeAreZero;
-			}
-
 			default:
 			{
 				int returnCode = WEXITSTATUS(status);
@@ -1175,31 +956,3 @@ copydb_collect_finished_subprocesses(bool *allDone)
 
 	return allReturnCodeAreZero;
 }
-
-
-/*
- * copydb_copy_roles copies roles from the source instance into the target
- * instance, using pg_dumpall --roles-only and our own SQL client that reads
- * the file and applies SQL command on the target system.
- */
-bool
-copydb_copy_roles(CopyDataSpec *copySpecs)
-{
-	if (!pg_dumpall_roles(&(copySpecs->pgPaths),
-						  copySpecs->source_pguri,
-						  copySpecs->dumpPaths.rolesFilename))
-	{
-		/* errors have already been logged */
-		return false;
-	}
-
-	if (!pg_restore_roles(&(copySpecs->pgPaths),
-						  copySpecs->target_pguri,
-						  copySpecs->dumpPaths.rolesFilename))
-	{
-		/* errors have already been logged */
-		return false;
-	}
-
-	return true;
-}
