@@ -9,7 +9,7 @@
 #include "git-version.h"
 
 /* additional version information for printing version on CLI */
-#define PGCOPYDB_VERSION "0.8"
+#define PGCOPYDB_VERSION "0.10"
 
 #ifdef GIT_VERSION
 #define VERSION_STRING GIT_VERSION
@@ -26,12 +26,18 @@
 /* environment variables for the command line options */
 #define PGCOPYDB_SOURCE_PGURI "PGCOPYDB_SOURCE_PGURI"
 #define PGCOPYDB_TARGET_PGURI "PGCOPYDB_TARGET_PGURI"
-#define PGCOPYDB_TARGET_TABLE_JOBS "PGCOPYDB_TARGET_TABLE_JOBS"
-#define PGCOPYDB_TARGET_INDEX_JOBS "PGCOPYDB_TARGET_INDEX_JOBS"
+#define PGCOPYDB_TABLE_JOBS "PGCOPYDB_TABLE_JOBS"
+#define PGCOPYDB_INDEX_JOBS "PGCOPYDB_INDEX_JOBS"
+#define PGCOPYDB_SPLIT_TABLES_LARGER_THAN "PGCOPYDB_SPLIT_TABLES_LARGER_THAN"
 #define PGCOPYDB_DROP_IF_EXISTS "PGCOPYDB_DROP_IF_EXISTS"
 #define PGCOPYDB_SNAPSHOT "PGCOPYDB_SNAPSHOT"
 #define PGCOPYDB_PGAPPNAME "pgcopydb"
 
+/* default values for the command line options */
+#define DEFAULT_TABLE_JOBS 4
+#define DEFAULT_INDEX_JOBS 4
+#define DEFAULT_SPLIT_TABLES_LARGER_THAN 0 /* no COPY partitioning by default */
+
 #define POSTGRES_CONNECT_TIMEOUT "10"
 
 
