@@ -136,6 +136,23 @@ copydb_target_prepare_schema(CopyDataSpec *specs)
 		return true;
 	}
 
+	/*
+	 * pg_restore --clean --if-exists gets easily confused when dealing with
+	 * partial schema information, such as when using only section=pre-data, or
+	 * when using the --use-list option as we do here.
+	 *
+	 * As a result, we implement --drop-if-exists our own way first, with a big
+	 * DROP IF EXISTS ... CASCADE statement that includes all our target tables.
+	 */
+	if (specs->restoreOptions.dropIfExists)
+	{
+		if (!copydb_target_drop_tables(specs))
+		{
+			/* errors have already been logged */
+			return false;
+		}
+	}
+
 	if (!pg_restore_db(&(specs->pgPaths),
 					   specs->target_pguri,
 					   specs->dumpPaths.preFilename,
@@ -158,6 +175,63 @@ copydb_target_prepare_schema(CopyDataSpec *specs)
 }
 
 
+/*
+ * copydb_target_drop_tables prepares and executes a SQL query that prepares
+ * our target database by means of a DROP IF EXISTS ... CASCADE statement that
+ * includes all our target tables.
+ */
+bool
+copydb_target_drop_tables(CopyDataSpec *specs)
+{
+	PQExpBuffer query = createPQExpBuffer();
+
+	log_info("Drop tables on the target database, per --drop-if-exists");
+
+	appendPQExpBuffer(query, "DROP TABLE IF EXISTS");
+
+	SourceTableArray *tableArray = &(specs->sourceTableArray);
+
+	for (int tableIndex = 0; tableIndex < tableArray->count; tableIndex++)
+	{
+		SourceTable *source = &(tableArray->array[tableIndex]);
+
+		appendPQExpBuffer(query, "%s \"%s\".\"%s\"",
+						  tableIndex == 0 ? " " : ", ",
+						  source->nspname,
+						  source->relname);
+	}
+
+	appendPQExpBuffer(query, "CASCADE");
+
+	/* memory allocation could have failed while building string */
+	if (PQExpBufferBroken(query))
+	{
+		log_error("Failed to create DROP IF EXISTS query: out of memory");
+		destroyPQExpBuffer(query);
+		return false;
+	}
+
+	PGSQL dst = { 0 };
+
+	if (!pgsql_init(&dst, specs->target_pguri, PGSQL_CONN_TARGET))
+	{
+		/* errors have already been logged */
+		destroyPQExpBuffer(query);
+		return false;
+	}
+
+	if (!pgsql_execute(&dst, query->data))
+	{
+		/* errors have already been logged */
+		return false;
+	}
+
+	destroyPQExpBuffer(query);
+
+	return true;
+}
+
+
 /*
  * copydb_target_finalize_schema finalizes the schema after all the data has
  * been copied over, and after indexes and their constraints have been created
@@ -289,21 +363,21 @@ copydb_write_restore_list(CopyDataSpec *specs, PostgresDumpSection section)
 		{
 			prefix = ";";
 
-			log_debug("Skipping already processed dumpId %d: %s %u %s",
-					  contents.array[i].dumpId,
-					  contents.array[i].desc,
-					  contents.array[i].objectOid,
-					  contents.array[i].restoreListName);
+			log_notice("Skipping already processed dumpId %d: %s %u %s",
+					   contents.array[i].dumpId,
+					   contents.array[i].desc,
+					   contents.array[i].objectOid,
+					   contents.array[i].restoreListName);
 		}
 		else if (copydb_objectid_is_filtered_out(specs, oid, name))
 		{
 			prefix = ";";
 
-			log_debug("Skipping filtered-out dumpId %d: %s %u %s",
-					  contents.array[i].dumpId,
-					  contents.array[i].desc,
-					  contents.array[i].objectOid,
-					  contents.array[i].restoreListName);
+			log_notice("Skipping filtered-out dumpId %d: %s %u %s",
+					   contents.array[i].dumpId,
+					   contents.array[i].desc,
+					   contents.array[i].objectOid,
+					   contents.array[i].restoreListName);
 		}
 
 		appendPQExpBuffer(listContents, "%s%d; %u %u %s %s\n",
