@@ -20,14 +20,14 @@
 #include "cli_root.h"
 #include "copydb.h"
 #include "env_utils.h"
+#include "ld_stream.h"
 #include "lock_utils.h"
 #include "log.h"
-#include "parsing.h"
+#include "parsing_utils.h"
 #include "pidfile.h"
 #include "pg_utils.h"
 #include "schema.h"
 #include "signals.h"
-#include "stream.h"
 #include "string_utils.h"
 #include "summary.h"
 
@@ -41,6 +41,234 @@ static bool streamLogicalTransactionAppendStatement(LogicalTransaction *txn,
 													);
 
 
+/*
+ * stream_transform_start_worker creates a sub-process that transform JSON
+ * files into SQL files as needed, consuming requests from a queue.
+ */
+bool
+stream_transform_start_worker(LogicalStreamContext *context)
+{
+	StreamContext *privateContext = (StreamContext *) context->private;
+
+	/*
+	 * Flush stdio channels just before fork, to avoid double-output
+	 * problems.
+	 */
+	fflush(stdout);
+	fflush(stderr);
+
+	int fpid = fork();
+
+	switch (fpid)
+	{
+		case -1:
+		{
+			log_error("Failed to fork a stream transform worker process: %m");
+			return false;
+		}
+
+		case 0:
+		{
+			/* child process runs the command */
+			if (!stream_transform_worker(context))
+			{
+				/* errors have already been logged */
+				exit(EXIT_CODE_INTERNAL_ERROR);
+			}
+
+			exit(EXIT_CODE_QUIT);
+		}
+
+		default:
+		{
+			/* fork succeeded, in parent */
+			privateContext->subprocess = fpid;
+			break;
+		}
+	}
+
+	return true;
+}
+
+
+/*
+ * stream_transform_worker is a worker process that loops over messages
+ * received from a queue, each message contains the WAL.json and the WAL.sql
+ * file names. When receiving such a message, the WAL.json file is transformed
+ * into the WAL.sql file.
+ */
+bool
+stream_transform_worker(LogicalStreamContext *context)
+{
+	StreamContext *privateContext = (StreamContext *) context->private;
+
+	int errors = 0;
+	bool stop = false;
+
+	log_notice("Started Stream Transform worker %d [%d]", getpid(), getppid());
+
+	while (!stop)
+	{
+		QMessage mesg = { 0 };
+
+		if (asked_to_stop || asked_to_stop_fast || asked_to_quit)
+		{
+			return false;
+		}
+
+		if (!queue_receive(&(privateContext->transformQueue), &mesg))
+		{
+			/* errors have already been logged */
+			break;
+		}
+
+		switch (mesg.type)
+		{
+			case QMSG_TYPE_STOP:
+			{
+				stop = true;
+				log_debug("Stop message received by stream transform worker");
+				break;
+			}
+
+			case QMSG_TYPE_STREAM_TRANSFORM:
+			{
+				log_debug("stream_transform_worker received transform %X/%X",
+						  LSN_FORMAT_ARGS(mesg.data.lsn));
+
+				if (!stream_compute_pathnames(context, mesg.data.lsn))
+				{
+					/* errors have already been logged, break from the loop */
+					++errors;
+					break;
+				}
+
+				if (!stream_transform_file(privateContext->walFileName,
+										   privateContext->sqlFileName))
+				{
+					/* errors have already been logged, break from the loop */
+					++errors;
+					break;
+				}
+				break;
+			}
+
+			default:
+			{
+				log_error("Received unknown message type %ld on vacuum queue %d",
+						  mesg.type,
+						  privateContext->transformQueue.qId);
+				break;
+			}
+		}
+	}
+
+	return stop == true && errors == 0;
+}
+
+
+/*
+ * stream_compute_pathnames computes the WAL.json and WAL.sql filenames from
+ * the given LSN, which is expected to be the first LSN processed in the file
+ * we need to find the name of.
+ */
+bool
+stream_compute_pathnames(LogicalStreamContext *context, uint64_t lsn)
+{
+	StreamContext *privateContext = (StreamContext *) context->private;
+
+	char wal[MAXPGPATH] = { 0 };
+
+	/*
+	 * The timeline and wal segment size are determined when connecting to the
+	 * source database, and stored to local files at that time. When the Stream
+	 * Transform Worker process is created, we don't have that information yet,
+	 * so the first time we process an LSN from the queue we go and fetch the
+	 * information from our local files.
+	 */
+	if (context->timeline == 0)
+	{
+		uint32_t WalSegSz;
+		IdentifySystem system = { 0 };
+
+		if (!stream_read_context(&(privateContext->paths), &system, &WalSegSz))
+		{
+			log_error("Failed to read the streaming context information "
+					  "from the source database, see above for details");
+			return false;
+		}
+
+		context->timeline = system.timeline;
+		context->WalSegSz = WalSegSz;
+	}
+
+	/* compute the WAL filename that would host the current LSN */
+	XLogSegNo segno;
+	XLByteToSeg(lsn, segno, context->WalSegSz);
+	XLogFileName(wal, context->timeline, segno, context->WalSegSz);
+
+	sformat(privateContext->walFileName,
+			sizeof(privateContext->walFileName),
+			"%s/%s.json",
+			privateContext->paths.dir,
+			wal);
+
+	sformat(privateContext->sqlFileName,
+			sizeof(privateContext->sqlFileName),
+			"%s/%s.sql",
+			privateContext->paths.dir,
+			wal);
+
+	return true;
+}
+
+
+/*
+ * vacuum_add_table sends a message to the VACUUM process queue to process
+ * given table.
+ */
+bool
+stream_transform_add_file(Queue *queue, uint64_t firstLSN)
+{
+	QMessage mesg = {
+		.type = QMSG_TYPE_STREAM_TRANSFORM,
+		.data.lsn = firstLSN
+	};
+
+	log_debug("stream_transform_add_file[%d]: %X/%X",
+			  queue->qId,
+			  LSN_FORMAT_ARGS(mesg.data.lsn));
+
+	if (!queue_send(queue, &mesg))
+	{
+		/* errors have already been logged */
+		return false;
+	}
+
+	return true;
+}
+
+
+/*
+ * vacuum_send_stop sends the STOP message to the Stream Transform worker.
+ */
+bool
+stream_transform_send_stop(Queue *queue)
+{
+	QMessage stop = { .type = QMSG_TYPE_STOP };
+
+	log_debug("Send STOP message to Transform Queue %d", queue->qId);
+
+	if (!queue_send(queue, &stop))
+	{
+		/* errors have already been logged */
+		return false;
+	}
+
+	return true;
+}
+
+
 /*
  * stream_transform_file transforms a JSON formatted file as received from the
  * wal2json logical decoding plugin into an SQL file ready for applying to the
@@ -52,6 +280,10 @@ stream_transform_file(char *jsonfilename, char *sqlfilename)
 	StreamContent content = { 0 };
 	long size = 0L;
 
+	log_notice("Transforming JSON file \"%s\" into SQL file \"%s\"",
+			   jsonfilename,
+			   sqlfilename);
+
 	strlcpy(content.filename, jsonfilename, sizeof(content.filename));
 
 	if (!read_file(content.filename, &(content.buffer), &size))
@@ -60,16 +292,13 @@ stream_transform_file(char *jsonfilename, char *sqlfilename)
 		return false;
 	}
 
-	content.count =
-		splitLines(content.buffer, content.lines, MAX_STREAM_CONTENT_COUNT);
+	content.count = countLines(content.buffer);
+	content.lines = (char **) calloc(content.count, sizeof(char *));
+	content.count = splitLines(content.buffer, content.lines, content.count);
 
-	if (content.count >= MAX_STREAM_CONTENT_COUNT)
+	if (content.lines == NULL)
 	{
-		log_error("Failed to split file \"%s\" in lines: pgcopydb support only "
-				  "files with up to %d lines, and more were found",
-				  content.filename,
-				  MAX_STREAM_CONTENT_COUNT);
-		free(content.buffer);
+		log_error(ALLOCATION_FAILED_ERROR);
 		return false;
 	}
 
@@ -77,6 +306,16 @@ stream_transform_file(char *jsonfilename, char *sqlfilename)
 			  content.count,
 			  content.filename);
 
+	content.messages =
+		(LogicalMessageMetadata *) calloc(content.count,
+										  sizeof(LogicalMessageMetadata));
+
+	if (content.messages == NULL)
+	{
+		log_error(ALLOCATION_FAILED_ERROR);
+		return false;
+	}
+
 	/* {action: B} {action: C} {action: X} */
 	int maxTxnsCount = (content.count / 2) + 1;
 	LogicalTransactionArray txns = { 0 };
@@ -159,6 +398,10 @@ stream_transform_file(char *jsonfilename, char *sqlfilename)
 		}
 	}
 
+	/* free dynamic memory that's not needed anymore */
+	free(content.lines);
+	free(content.messages);
+
 	log_debug("stream_transform_file read %d transactions", txns.count);
 
 	/*
@@ -193,6 +436,9 @@ stream_transform_file(char *jsonfilename, char *sqlfilename)
 		return false;
 	}
 
+	log_info("Transformed JSON messages into SQL file \"%s\"",
+			 sqlfilename);
+
 	return true;
 }
 
@@ -231,15 +477,22 @@ parseMessage(LogicalTransaction *txn,
 		return false;
 	}
 
-	if (txn->xid > 0 && txn->xid != metadata->xid)
+	/*
+	 * Check that XID make sense, except for SWITCH messages, which don't have
+	 * XID information, only have LSN information.
+	 */
+	if (metadata->action != STREAM_ACTION_SWITCH)
 	{
-		log_debug("%s", message);
-		log_error("BUG: logical message xid is %lld, which is different "
-				  "from the current transaction xid %lld",
-				  (long long) metadata->xid,
-				  (long long) txn->xid);
+		if (txn->xid > 0 && txn->xid != metadata->xid)
+		{
+			log_debug("%s", message);
+			log_error("BUG: logical message xid is %lld, which is different "
+					  "from the current transaction xid %lld",
+					  (long long) metadata->xid,
+					  (long long) txn->xid);
 
-		return false;
+			return false;
+		}
 	}
 
 	/* common entries for supported statements */
@@ -818,7 +1071,7 @@ stream_write_insert(FILE *out, LogicalMessageInsert *insert)
 		fformat(out, "(");
 		for (int c = 0; c < stmt->cols; c++)
 		{
-			fformat(out, "%s%s", c > 0 ? ", " : "", stmt->columns[c]);
+			fformat(out, "%s\"%s\"", c > 0 ? ", " : "", stmt->columns[c]);
 		}
 		fformat(out, ")");
 
