@@ -18,7 +18,7 @@
 #include "env_utils.h"
 #include "file_utils.h"
 #include "log.h"
-#include "parsing.h"
+#include "parsing_utils.h"
 #include "pgcmd.h"
 #include "signals.h"
 #include "string_utils.h"
@@ -682,6 +682,33 @@ pg_restore_roles(PostgresPaths *pgPaths,
 }
 
 
+/*
+ * pg_copy_roles copies roles from the source instance into the target
+ * instance, using pg_dumpall --roles-only and our own SQL client that reads
+ * the file and applies SQL command on the target system.
+ */
+bool
+pg_copy_roles(PostgresPaths *pgPaths,
+			  const char *source_pguri,
+			  const char *target_pguri,
+			  const char *filename)
+{
+	if (!pg_dumpall_roles(pgPaths, source_pguri, filename))
+	{
+		/* errors have already been logged */
+		return false;
+	}
+
+	if (!pg_restore_roles(pgPaths, target_pguri, filename))
+	{
+		/* errors have already been logged */
+		return false;
+	}
+
+	return true;
+}
+
+
 /*
  * Call pg_restore from the given filename and restores it to the target
  * database connection.
@@ -814,7 +841,7 @@ pg_restore_list(PostgresPaths *pgPaths, const char *filename,
 	char command[BUFSIZE] = { 0 };
 	(void) snprintf_program_command_line(&prog, command, BUFSIZE);
 
-	log_debug("%s", command);
+	log_notice("%s", command);
 
 	if (prog.returnCode != 0)
 	{
@@ -863,17 +890,17 @@ struct StringWithLength
  * postgres/src/bin/pg_dump/pg_dump_sort.c
  */
 struct StringWithLength pgRestoreDescriptionArray[] = {
-	INSERT_STRING_WITH_LENGTH("ACL"),
 	INSERT_STRING_WITH_LENGTH("ACCESS METHOD"),
+	INSERT_STRING_WITH_LENGTH("ACL"),
 	INSERT_STRING_WITH_LENGTH("AGGREGATE"),
 	INSERT_STRING_WITH_LENGTH("ATTRDEF"),
 	INSERT_STRING_WITH_LENGTH("BLOB DATA"),
 	INSERT_STRING_WITH_LENGTH("BLOB"),
 	INSERT_STRING_WITH_LENGTH("CAST"),
 	INSERT_STRING_WITH_LENGTH("COLLATION"),
+	INSERT_STRING_WITH_LENGTH("COMMENT"),
 	INSERT_STRING_WITH_LENGTH("CONSTRAINT"),
 	INSERT_STRING_WITH_LENGTH("CONVERSION"),
-	INSERT_STRING_WITH_LENGTH("COMMENT"),
 	INSERT_STRING_WITH_LENGTH("DEFAULT ACL"),
 	INSERT_STRING_WITH_LENGTH("DEFAULT"),
 	INSERT_STRING_WITH_LENGTH("DOMAIN"),
@@ -883,16 +910,19 @@ struct StringWithLength pgRestoreDescriptionArray[] = {
 	INSERT_STRING_WITH_LENGTH("FK CONSTRAINT"),
 	INSERT_STRING_WITH_LENGTH("FOREIGN DATA WRAPPER"),
 	INSERT_STRING_WITH_LENGTH("FOREIGN SERVER"),
+	INSERT_STRING_WITH_LENGTH("FOREIGN TABLE"),
 	INSERT_STRING_WITH_LENGTH("FUNCTION"),
 	INSERT_STRING_WITH_LENGTH("INDEX ATTACH"),
 	INSERT_STRING_WITH_LENGTH("INDEX"),
+	INSERT_STRING_WITH_LENGTH("MATERIALIZED VIEW"),
 	INSERT_STRING_WITH_LENGTH("OPERATOR CLASS"),
 	INSERT_STRING_WITH_LENGTH("OPERATOR FAMILY"),
 	INSERT_STRING_WITH_LENGTH("OPERATOR"),
 	INSERT_STRING_WITH_LENGTH("POLICY"),
 	INSERT_STRING_WITH_LENGTH("PROCEDURAL LANGUAGE"),
-	INSERT_STRING_WITH_LENGTH("PUBLICATION TABLE"),
+	INSERT_STRING_WITH_LENGTH("PROCEDURE"),
 	INSERT_STRING_WITH_LENGTH("PUBLICATION TABLES IN SCHEMA"),
+	INSERT_STRING_WITH_LENGTH("PUBLICATION TABLE"),
 	INSERT_STRING_WITH_LENGTH("PUBLICATION"),
 	INSERT_STRING_WITH_LENGTH("REFRESH MATERIALIZED VIEW"),
 	INSERT_STRING_WITH_LENGTH("RULE"),
@@ -900,6 +930,7 @@ struct StringWithLength pgRestoreDescriptionArray[] = {
 	INSERT_STRING_WITH_LENGTH("SEQUENCE OWNED BY"),
 	INSERT_STRING_WITH_LENGTH("SEQUENCE SET"),
 	INSERT_STRING_WITH_LENGTH("SEQUENCE"),
+	INSERT_STRING_WITH_LENGTH("SERVER"),
 	INSERT_STRING_WITH_LENGTH("SHELL TYPE"),
 	INSERT_STRING_WITH_LENGTH("STATISTICS"),
 	INSERT_STRING_WITH_LENGTH("SUBSCRIPTION"),
@@ -913,6 +944,7 @@ struct StringWithLength pgRestoreDescriptionArray[] = {
 	INSERT_STRING_WITH_LENGTH("TRANSFORM"),
 	INSERT_STRING_WITH_LENGTH("TRIGGER"),
 	INSERT_STRING_WITH_LENGTH("TYPE"),
+	INSERT_STRING_WITH_LENGTH("USER MAPPING"),
 	INSERT_STRING_WITH_LENGTH("VIEW"),
 	{ 0, "" }
 };
@@ -1002,14 +1034,31 @@ parse_archive_list(char *list, ArchiveContentArray *contents)
 						pgRestoreDescriptionArray[i].str,
 						pgRestoreDescriptionArray[i].len) == 0)
 			{
-				strlcpy(item->desc,
-						pgRestoreDescriptionArray[i].str,
-						sizeof(item->desc));
-
-				strlcpy(item->restoreListName,
-						ptr + pgRestoreDescriptionArray[i].len + 1,
-						sizeof(item->restoreListName));
+				/*
+				 * Some pg_restore archive catalog TOC entries have a quite a
+				 * special restoreListName, that needs some tweaking to be able
+				 * to match it to the normal one we have in our hash tables.
+				 *
+				 */
+				if (strcmp(pgRestoreDescriptionArray[i].str, "ACL") == 0 ||
+					strcmp(pgRestoreDescriptionArray[i].str, "COMMENT") == 0)
+				{
+					/* ignore errors */
+					if (!parse_archive_acl_or_comment(ptr, item))
+					{
+						log_debug("Failed to parse ACL or COMMENT: %s", ptr);
+					}
+				}
+				else
+				{
+					strlcpy(item->desc,
+							pgRestoreDescriptionArray[i].str,
+							sizeof(item->desc));
 
+					strlcpy(item->restoreListName,
+							ptr + pgRestoreDescriptionArray[i].len + 1,
+							sizeof(item->restoreListName));
+				}
 				break;
 			}
 		}
@@ -1024,3 +1073,120 @@ parse_archive_list(char *list, ArchiveContentArray *contents)
 
 	return true;
 }
+
+
+/*
+ * parse_archive_acl_or_comment parses the ACL or COMMENT entry of the
+ * pg_restore archive catalog TOC.
+ *
+ * 4837; 0 0 ACL - SCHEMA public postgres
+ * 4838; 0 0 COMMENT - SCHEMA topology dim
+ * 4839; 0 0 COMMENT - EXTENSION intarray
+ * 4840; 0 0 COMMENT - EXTENSION postgis
+ *
+ * Here the - is for the namespace, which doesn't apply, and then the TAG is
+ * composite: TYPE name; where it usually is just the object name.
+ *
+ * The ptr argument is positioned at the start of either ACL or COMMENT.
+ */
+bool
+parse_archive_acl_or_comment(char *ptr, ArchiveContentItem *item)
+{
+	log_trace("parse_archive_acl_or_comment: %s", ptr);
+
+	char *desc = ptr;
+
+	/* skip "ACL " or "COMMENT " */
+	char *aclPrefix = "ACL ";
+	size_t aclPrefixLen = strlen(aclPrefix);
+
+	char *commentPrefix = "COMMENT ";
+	size_t commentPrefixLen = strlen(commentPrefix);
+
+	if (strncmp(ptr, aclPrefix, aclPrefixLen) == 0)
+	{
+		ptr += aclPrefixLen;
+		strlcpy(item->desc, "ACL", sizeof(item->desc));
+	}
+	else if (strncmp(ptr, commentPrefix, commentPrefixLen) == 0)
+	{
+		ptr += commentPrefixLen;
+		strlcpy(item->desc, "COMMENT", sizeof(item->desc));
+	}
+	else
+	{
+		log_warn("Failed to parse desc \"%s\" for ACL or COMMENT", desc);
+		return false;
+	}
+
+	/*
+	 * At the moment we only support filtering ACLs and COMMENTS for SCHEMA and
+	 * EXTENSION objects, see --skip-extensions. So first, we skil the
+	 * namespace, which in our case would always be a dash.
+	 */
+
+	char nspname[NAMEDATALEN];
+
+	char *space = strchr(ptr, ' ');
+
+	if (space != NULL)
+	{
+		*space = '\0';
+		strlcpy(nspname, ptr, sizeof(nspname));
+
+		ptr = space + 1;
+	}
+	else
+	{
+		log_warn("Failed to parse desc \"%s\" for ACL or COMMENT", desc);
+		return false;
+	}
+
+	char *SCHEMA = "SCHEMA ";
+	size_t SCHEMA_LEN = strlen(SCHEMA);
+
+	char *EXTENSION = "EXTENSION ";
+	size_t EXTENSION_LEN = strlen(EXTENSION);
+
+	if (strncmp(ptr, SCHEMA, SCHEMA_LEN) == 0)
+	{
+		/* a schema pg_restore list name is "- nspname rolname" */
+		char *nsp_rol_name = ptr + SCHEMA_LEN;
+
+		sformat(item->restoreListName, sizeof(item->restoreListName),
+				"- %s",
+				nsp_rol_name);
+	}
+	else if (strncmp(ptr, EXTENSION, EXTENSION_LEN) == 0)
+	{
+		/*
+		 * The extension name is following by a space, even though there is no
+		 * owner to follow that space. We don't want that space at the end of
+		 * the extension's name.
+		 */
+		char *extname = ptr + EXTENSION_LEN;
+		char *space = strchr(extname, ' ');
+		*space = '\0';
+
+		/* an extension's pg_restore list name is just its name */
+		sformat(item->restoreListName, sizeof(item->restoreListName),
+				"%s",
+				extname);
+	}
+	else
+	{
+		char *sep = strchr(ptr, ' ');
+		*sep = '\0';
+
+		log_debug("Failed to parse %s for %s: not supported yet",
+				  item->desc,
+				  ptr);
+		return false;
+	}
+
+	log_trace("parse_archive_acl_or_comment: %s [%s]",
+			  item->desc,
+			  item->restoreListName);
+
+	return true;
+}
