@@ -39,7 +39,8 @@ bool stringToUInt32(const char *str, uint32_t *number);
 bool stringToDouble(const char *str, double *number);
 bool IntervalToString(uint64_t millisecs, char *buffer, size_t size);
 
-int splitLines(char *errorMessage, char **linesArray, int size);
+int countLines(char *buffer);
+int splitLines(char *buffer, char **linesArray, int size);
 void processBufferCallback(const char *buffer, bool error);
 
 void pretty_print_bytes(char *buffer, size_t size, uint64_t bytes);
