@@ -7,6 +7,8 @@
 #include <getopt.h>
 #include <inttypes.h>
 
+#include "parson.h"
+
 #include "copydb.h"
 #include "env_utils.h"
 #include "log.h"
@@ -43,7 +45,58 @@ write_table_summary(CopyTableSummary *summary, char *filename)
 			summary->command);
 
 	/* write the summary to the doneFile */
-	return write_file(contents, strlen(contents), filename);
+	if (!write_file(contents, strlen(contents), filename))
+	{
+		log_error("Failed to write table summary file \"%s\"", filename);
+		return false;
+	}
+
+	return true;
+}
+
+
+/*
+ * prepare_table_summary_as_json prepares the summary information as a JSON
+ * object within the given JSON_Value.
+ */
+bool
+prepare_table_summary_as_json(CopyTableSummary *summary,
+							  JSON_Object *jsobj,
+							  const char *key)
+{
+	JSON_Value *jsSummary = json_value_init_object();
+	JSON_Object *jsSummaryObj = json_value_get_object(jsSummary);
+
+	json_object_set_number(jsSummaryObj, "pid", (double) summary->pid);
+
+	json_object_set_number(jsSummaryObj,
+						   "start-time-epoch",
+						   (double) summary->startTime);
+
+	/* pretty print start time */
+	time_t secs = summary->startTime;
+	struct tm ts = { 0 };
+	char startTimeStr[BUFSIZE] = { 0 };
+
+	if (localtime_r(&secs, &ts) == NULL)
+	{
+		log_error("Failed to convert seconds %lld to local time: %m",
+				  (long long) secs);
+		return false;
+	}
+
+	strftime(startTimeStr, sizeof(startTimeStr), "%Y-%m-%d %H:%M:%S %Z", &ts);
+
+	json_object_set_string(jsSummaryObj,
+						   "start-time-string",
+						   startTimeStr);
+
+	json_object_set_string(jsSummaryObj, "command", summary->command);
+
+	/* attach the JSON array to the main JSON object under the provided key */
+	json_object_set_value(jsobj, key, jsSummary);
+
+	return true;
 }
 
 
@@ -190,15 +243,8 @@ finish_table_summary(CopyTableSummary *summary, char *filename)
  * index doneFile.
  */
 bool
-create_table_index_file(CopyTableSummary *summary,
-						SourceIndexArray *indexArray,
-						char *filename)
+create_table_index_file(SourceTable *table, char *filename)
 {
-	if (indexArray->count < 1)
-	{
-		return true;
-	}
-
 	PQExpBuffer content = createPQExpBuffer();
 
 	if (content == NULL)
@@ -208,9 +254,11 @@ create_table_index_file(CopyTableSummary *summary,
 		return false;
 	}
 
-	for (int i = 0; i < indexArray->count; i++)
+	SourceIndexList *indexListEntry = table->firstIndex;
+
+	for (; indexListEntry != NULL; indexListEntry = indexListEntry->next)
 	{
-		SourceIndex *index = &(indexArray->array[i]);
+		SourceIndex *index = indexListEntry->index;
 
 		appendPQExpBuffer(content, "%d\n", index->indexOid);
 		appendPQExpBuffer(content, "%d\n", index->constraintOid);
@@ -224,10 +272,14 @@ create_table_index_file(CopyTableSummary *summary,
 		return false;
 	}
 
-	bool success = write_file(content->data, content->len, filename);
-	destroyPQExpBuffer(content);
+	if (!write_file(content->data, content->len, filename))
+	{
+		log_error("Failed to write file \"%s\"", filename);
+		destroyPQExpBuffer(content);
+		return false;
+	}
 
-	return success;
+	return true;
 }
 
 
@@ -258,15 +310,23 @@ read_table_index_file(SourceIndexArray *indexArray, char *filename)
 	char *fileLines[BUFSIZE] = { 0 };
 	int lineCount = splitLines(fileContents, fileLines, BUFSIZE);
 
-	/* we expect to have an indexOid per line, no comments, etc */
-	indexArray->count = lineCount;
-	indexArray->array = (SourceIndex *) malloc(lineCount * sizeof(SourceIndex));
+	/*
+	 * We expect to have alternate lines with first indexOid and then
+	 * constraintOid (which could be zero). No comments, etc.
+	 */
+	indexArray->count = lineCount / 2;
+	indexArray->array = (SourceIndex *) calloc(lineCount, sizeof(SourceIndex));
+
+	int pos = 0;
 
 	for (int i = 0; i < lineCount; i++)
 	{
-		SourceIndex *index = &(indexArray->array[i]);
+		SourceIndex *index = &(indexArray->array[pos]);
 
-		if (!stringToUInt32(fileLines[i], &(index->indexOid)))
+		uint32_t *target =
+			i % 2 == 0 ? &(index->indexOid) : &(index->constraintOid);
+
+		if (!stringToUInt32(fileLines[i], target))
 		{
 			log_error("Failed to read the index oid \"%s\" "
 					  "in file \"%s\" at line %d",
@@ -275,6 +335,12 @@ read_table_index_file(SourceIndexArray *indexArray, char *filename)
 					  i);
 			return false;
 		}
+
+		/* one index entry (indexOid, constraintOid) spans two lines */
+		if (i % 2 == 1)
+		{
+			pos++;
+		}
 	}
 
 	return true;
@@ -405,6 +471,49 @@ read_index_summary(CopyIndexSummary *summary, const char *filename)
 }
 
 
+/*
+ * prepare_index_summary_as_json prepares the summary information as a JSON
+ * object within the given JSON_Value.
+ */
+bool
+prepare_index_summary_as_json(CopyIndexSummary *summary,
+							  JSON_Object *jsobj,
+							  const char *key)
+{
+	JSON_Value *jsSummary = json_value_init_object();
+	JSON_Object *jsSummaryObj = json_value_get_object(jsSummary);
+
+	json_object_set_number(jsSummaryObj, "pid", (double) summary->pid);
+
+	json_object_set_number(jsSummaryObj,
+						   "start-time-epoch",
+						   (double) summary->startTime);
+
+	/* pretty print start time */
+	time_t secs = summary->startTime;
+	struct tm ts = { 0 };
+	char startTimeStr[BUFSIZE] = { 0 };
+
+	if (localtime_r(&secs, &ts) == NULL)
+	{
+		log_error("Failed to convert seconds %lld to local time: %m",
+				  (long long) secs);
+		return false;
+	}
+
+	strftime(startTimeStr, sizeof(startTimeStr), "%Y-%m-%d %H:%M:%S %Z", &ts);
+
+	json_object_set_string(jsSummaryObj,
+						   "start-time-string",
+						   startTimeStr);
+
+	/* attach the JSON array to the main JSON object under the provided key */
+	json_object_set_value(jsobj, key, jsSummary);
+
+	return true;
+}
+
+
 /*
  * open_index_summary initializes the time elements of an index summary and
  * writes the summary in the given filename. Typically, the lockFile.
@@ -537,6 +646,12 @@ summary_set_current_time(TopLevelTimings *timings, TimingStep step)
 			break;
 		}
 
+		case TIMING_STEP_BEFORE_SCHEMA_FETCH:
+		{
+			INSTR_TIME_SET_CURRENT(timings->beforeSchemaFetch);
+			break;
+		}
+
 		case TIMING_STEP_BEFORE_SCHEMA_DUMP:
 		{
 			INSTR_TIME_SET_CURRENT(timings->beforeSchemaDump);
@@ -593,7 +708,7 @@ summary_prepare_toplevel_durations(Summary *summary)
 	uint64_t durationMs;
 
 	/* compute schema dump duration, part of schemaDurationMs */
-	duration = timings->beforePrepareSchema;
+	duration = timings->beforeSchemaFetch;
 	INSTR_TIME_SUBTRACT(duration, timings->beforeSchemaDump);
 	durationMs = INSTR_TIME_MS(duration);
 
@@ -601,6 +716,15 @@ summary_prepare_toplevel_durations(Summary *summary)
 
 	timings->schemaDurationMs = durationMs;
 
+	/* compute schema fetch duration, part of schemaDurationMs */
+	duration = timings->beforePrepareSchema;
+	INSTR_TIME_SUBTRACT(duration, timings->beforeSchemaFetch);
+	durationMs = INSTR_TIME_MS(duration);
+
+	IntervalToString(durationMs, timings->fetchSchemaMs, INTSTRING_MAX_DIGITS);
+
+	timings->schemaDurationMs += durationMs;
+
 	/* compute prepare schema duration, part of schemaDurationMs */
 	duration = timings->afterPrepareSchema;
 	INSTR_TIME_SUBTRACT(duration, timings->beforePrepareSchema);
@@ -656,55 +780,63 @@ print_toplevel_summary(Summary *summary, int tableJobs, int indexJobs)
 {
 	char *d10s = "----------";
 	char *d12s = "------------";
-	char *d45s = "---------------------------------------------";
+	char *d50s = "--------------------------------------------------";
 
 	fformat(stdout, "\n");
 
-	fformat(stdout, " %45s   %10s  %10s  %12s\n",
+	fformat(stdout, " %50s   %10s  %10s  %12s\n",
 			"Step", "Connection", "Duration", "Concurrency");
 
-	fformat(stdout, " %45s   %10s  %10s  %12s\n", d45s, d10s, d10s, d12s);
+	fformat(stdout, " %50s   %10s  %10s  %12s\n", d50s, d10s, d10s, d12s);
 
-	fformat(stdout, " %45s   %10s  %10s  %12d\n", "Dump Schema", "source",
+	fformat(stdout, " %50s   %10s  %10s  %12d\n", "Dump Schema", "source",
 			summary->timings.dumpSchemaMs, 1);
 
-	fformat(stdout, " %45s   %10s  %10s  %12d\n", "Prepare Schema", "target",
+	fformat(stdout, " %50s   %10s  %10s  %12d\n",
+			"Catalog Queries (table ordering, filtering, etc)",
+			"source",
+			summary->timings.fetchSchemaMs,
+			1);
+
+	fformat(stdout, " %50s   %10s  %10s  %12d\n", "Prepare Schema", "target",
 			summary->timings.prepareSchemaMs, 1);
 
 	char concurrency[BUFSIZE] = { 0 };
-	sformat(concurrency, sizeof(concurrency), "%d + %d", tableJobs, indexJobs);
+	sformat(concurrency, sizeof(concurrency), "%d + %d",
+			tableJobs,
+			tableJobs + indexJobs);
 
-	fformat(stdout, " %45s   %10s  %10s  %12s\n",
+	fformat(stdout, " %50s   %10s  %10s  %12s\n",
 			"COPY, INDEX, CONSTRAINTS, VACUUM (wall clock)", "both",
 			summary->timings.dataAndIndexMs,
 			concurrency);
 
-	fformat(stdout, " %45s   %10s  %10s  %12d\n",
+	fformat(stdout, " %50s   %10s  %10s  %12d\n",
 			"COPY (cumulative)", "both",
 			summary->timings.totalTableMs,
 			tableJobs);
 
-	fformat(stdout, " %45s   %10s  %10s  %12d\n",
+	fformat(stdout, " %50s   %10s  %10s  %12d\n",
 			"Large Objects (cumulative)", "both",
 			summary->timings.blobsMs,
 			1);
 
-	fformat(stdout, " %45s   %10s  %10s  %12d\n",
+	fformat(stdout, " %50s   %10s  %10s  %12d\n",
 			"CREATE INDEX, CONSTRAINTS (cumulative)", "target",
 			summary->timings.totalIndexMs,
 			indexJobs);
 
-	fformat(stdout, " %45s   %10s  %10s  %12d\n", "Finalize Schema", "target",
+	fformat(stdout, " %50s   %10s  %10s  %12d\n", "Finalize Schema", "target",
 			summary->timings.finalizeSchemaMs, 1);
 
-	fformat(stdout, " %45s   %10s  %10s  %12s\n", d45s, d10s, d10s, d12s);
+	fformat(stdout, " %50s   %10s  %10s  %12s\n", d50s, d10s, d10s, d12s);
 
-	fformat(stdout, " %45s   %10s  %10s  %12s\n",
+	fformat(stdout, " %50s   %10s  %10s  %12s\n",
 			"Total Wall Clock Duration", "both",
 			summary->timings.totalMs,
 			concurrency);
 
-	fformat(stdout, " %45s   %10s  %10s  %12s\n", d45s, d10s, d10s, d12s);
+	fformat(stdout, " %50s   %10s  %10s  %12s\n", d50s, d10s, d10s, d12s);
 
 	fformat(stdout, "\n");
 }
@@ -914,7 +1046,7 @@ prepare_summary_table(Summary *summary, CopyDataSpec *specs)
 	for (int tableIndex = 0; tableIndex < tableSpecsArray->count; tableIndex++)
 	{
 		CopyTableDataSpec *tableSpecs = &(tableSpecsArray->array[tableIndex]);
-		SourceTable *table = &(tableSpecs->sourceTable);
+		SourceTable *table = tableSpecs->sourceTable;
 
 		SummaryTableEntry *entry = &(summaryTable->array[tableIndex]);
 
@@ -945,37 +1077,65 @@ prepare_summary_table(Summary *summary, CopyDataSpec *specs)
 
 		SourceIndexArray indexArray = { 0 };
 
-		if (!read_table_index_file(&indexArray,
-								   tableSpecs->tablePaths.idxListFile))
+		/*
+		 * When the table COPY processing was split into several processes,
+		 * ensure we only read the index list once: only one of those COPY
+		 * processes started the indexing.
+		 */
+		if (tableSpecs->part.partNumber == 0)
 		{
-			/* errors have already been logged */
-			return false;
-		}
-
-		/* for reach index, read the index summary */
-		for (int i = 0; i < indexArray.count; i++)
-		{
-			SourceIndex *index = &(indexArray.array[i]);
-			CopyIndexSummary indexSummary = { .index = index };
-			char indexDoneFile[MAXPGPATH] = { 0 };
-
-			/* build the indexDoneFile for the target index */
-			sformat(indexDoneFile, sizeof(indexDoneFile), "%s/%u.done",
-					specs->cfPaths.idxdir,
-					index->indexOid);
-
-			/* when a table has no indexes, the file doesn't exists */
-			if (file_exists(indexDoneFile))
+			if (!read_table_index_file(&indexArray,
+									   tableSpecs->tablePaths.idxListFile))
 			{
-				if (!read_index_summary(&indexSummary, indexDoneFile))
+				/* errors have already been logged */
+				return false;
+			}
+
+			/* for reach index, read the index summary */
+			for (int i = 0; i < indexArray.count; i++)
+			{
+				SourceIndex *index = &(indexArray.array[i]);
+
+				CopyFilePaths *cfPaths = &(specs->cfPaths);
+				IndexFilePaths indexPaths = { 0 };
+
+				if (!copydb_init_index_paths(cfPaths, index, &indexPaths))
 				{
 					/* errors have already been logged */
 					return false;
 				}
 
-				/* accumulate total duration of creating all the indexes */
-				timings->indexDurationMs += indexSummary.durationMs;
-				indexingDurationMs += indexSummary.durationMs;
+				/* when a table has no indexes, the file doesn't exists */
+				if (file_exists(indexPaths.doneFile))
+				{
+					CopyIndexSummary indexSummary = { .index = index };
+
+					if (!read_index_summary(&indexSummary, indexPaths.doneFile))
+					{
+						/* errors have already been logged */
+						return false;
+					}
+
+					/* accumulate total duration of creating all the indexes */
+					timings->indexDurationMs += indexSummary.durationMs;
+					indexingDurationMs += indexSummary.durationMs;
+				}
+
+				if (file_exists(indexPaths.constraintDoneFile))
+				{
+					CopyIndexSummary indexSummary = { .index = index };
+
+					if (!read_index_summary(&indexSummary,
+											indexPaths.constraintDoneFile))
+					{
+						/* errors have already been logged */
+						return false;
+					}
+
+					/* accumulate total duration of creating all the indexes */
+					timings->indexDurationMs += indexSummary.durationMs;
+					indexingDurationMs += indexSummary.durationMs;
+				}
 			}
 		}
 
