@@ -7,8 +7,8 @@ set -e
 #
 #  - PGCOPYDB_SOURCE_PGURI
 #  - PGCOPYDB_TARGET_PGURI
-#  - PGCOPYDB_TARGET_TABLE_JOBS
-#  - PGCOPYDB_TARGET_INDEX_JOBS
+#  - PGCOPYDB_TABLE_JOBS
+#  - PGCOPYDB_INDEX_JOBS
 
 
 #
@@ -18,8 +18,8 @@ set -e
 pgcopydb list tables --source ${PGCOPYDB_SOURCE_PGURI}
 pgcopydb list tables --source ${PGCOPYDB_TARGET_PGURI}
 
-psql -d ${PGCOPYDB_SOURCE_PGURI} -1 -f /usr/src/pagila/pagila-schema.sql
-psql -d ${PGCOPYDB_SOURCE_PGURI} -1 -f /usr/src/pagila/pagila-data.sql
+psql -o /tmp/s.out -d ${PGCOPYDB_SOURCE_PGURI} -1 -f /usr/src/pagila/pagila-schema.sql
+psql -o /tmp/d.out -d ${PGCOPYDB_SOURCE_PGURI} -1 -f /usr/src/pagila/pagila-data.sql
 
 # pgcopydb clone uses the environment variables
 pgcopydb clone --filters /usr/src/pgcopydb/include.ini
