@@ -7,8 +7,8 @@ set -e
 #
 #  - PGCOPYDB_SOURCE_PGURI
 #  - PGCOPYDB_TARGET_PGURI
-#  - PGCOPYDB_TARGET_TABLE_JOBS
-#  - PGCOPYDB_TARGET_INDEX_JOBS
+#  - PGCOPYDB_TABLE_JOBS
+#  - PGCOPYDB_INDEX_JOBS
 
 #
 # pgcopydb list tables include a retry loop, so we use that as a proxy to
@@ -17,8 +17,12 @@ set -e
 pgcopydb list tables --source ${PGCOPYDB_SOURCE_PGURI}
 pgcopydb list tables --source ${PGCOPYDB_TARGET_PGURI}
 
-psql -d ${PGCOPYDB_SOURCE_PGURI} -1 -f /usr/src/pagila/pagila-schema.sql
-psql -d ${PGCOPYDB_SOURCE_PGURI} -1 -f /usr/src/pagila/pagila-data.sql
+psql -o /tmp/d.out -d ${PGCOPYDB_SOURCE_PGURI} -1 -f /usr/src/pagila/pagila-schema.sql
+psql -o /tmp/s.out -d ${PGCOPYDB_SOURCE_PGURI} -1 -f /usr/src/pagila/pagila-data.sql
+
+psql -d ${PGCOPYDB_TARGET_PGURI} <<EOF
+alter database postgres connection limit 2;
+EOF
 
 #
 # pgcopydb uses the environment variables
