@@ -7,8 +7,8 @@ set -e
 #
 #  - PGCOPYDB_SOURCE_PGURI
 #  - PGCOPYDB_TARGET_PGURI
-#  - PGCOPYDB_TARGET_TABLE_JOBS
-#  - PGCOPYDB_TARGET_INDEX_JOBS
+#  - PGCOPYDB_TABLE_JOBS
+#  - PGCOPYDB_INDEX_JOBS
 
 
 #
@@ -21,15 +21,15 @@ pgcopydb list tables --source ${PGCOPYDB_TARGET_PGURI}
 #
 # Now create a user that's going to be the owner of our database
 #
-psql ${PGCOPYDB_SOURCE_PGURI} <<EOF
+psql -a ${PGCOPYDB_SOURCE_PGURI} <<EOF
 create role pagila NOSUPERUSER CREATEDB NOCREATEROLE LOGIN PASSWORD '0wn3d';
-create database pagila owner pagila;
+create database pagila owner pagila connection limit 8;
 EOF
 
 pgcopydb copy roles
 
-psql ${PGCOPYDB_TARGET_PGURI} <<EOF
-create database pagila owner pagila;
+psql -a ${PGCOPYDB_TARGET_PGURI} <<EOF
+create database pagila owner pagila connection limit 10;
 EOF
 
 PAGILA_SOURCE_PGURI="postgres://pagila:0wn3d@source/pagila"
@@ -37,8 +37,7 @@ PAGILA_TARGET_PGURI="postgres://pagila:0wn3d@target/pagila"
 
 grep -v "OWNER TO postgres" /usr/src/pagila/pagila-schema.sql > /tmp/pagila-schema.sql
 
-psql -d ${PAGILA_SOURCE_PGURI} -1 -f /tmp/pagila-schema.sql
-psql -d ${PAGILA_SOURCE_PGURI} -1 -f /usr/src/pagila/pagila-data.sql
+psql -o /tmp/s.out -d ${PAGILA_SOURCE_PGURI} -1 -f /tmp/pagila-schema.sql
+psql -o /tmp/d.out -d ${PAGILA_SOURCE_PGURI} -1 -f /usr/src/pagila/pagila-data.sql
 
-# pgcopydb clone uses the environment variables
 pgcopydb clone --source ${PAGILA_SOURCE_PGURI} --target ${PAGILA_TARGET_PGURI}
