@@ -7,8 +7,8 @@ set -e
 #
 #  - PGCOPYDB_SOURCE_PGURI
 #  - PGCOPYDB_TARGET_PGURI
-#  - PGCOPYDB_TARGET_TABLE_JOBS
-#  - PGCOPYDB_TARGET_INDEX_JOBS
+#  - PGCOPYDB_TABLE_JOBS
+#  - PGCOPYDB_INDEX_JOBS
 
 #
 # pgcopydb list tables include a retry loop, so we use that as a proxy to
@@ -16,7 +16,7 @@ set -e
 #
 pgcopydb list tables --source "${PGCOPYDB_SOURCE_PGURI}"
 
-psql -d "${PGCOPYDB_SOURCE_PGURI}" -1 -f ./setup/setup.sql
+psql -q -d "${PGCOPYDB_SOURCE_PGURI}" -1 -f ./setup/setup.sql
 
 # pgcopydb fork uses the environment variables
 pgcopydb fork
